from smart_module.module_system.header_sounds import *

sounds = [
 ("click", sf_2d|sf_vol_3,["drum_3.ogg"]),
 ("tutorial_1", sf_2d|sf_vol_7,["tutorial_1.ogg"]),
 ("tutorial_2", sf_2d|sf_vol_7,["tutorial_2.ogg"]),
 ("gong", sf_2d|sf_priority_9|sf_vol_7, ["s_cymbals.ogg"]),
 ("quest_taken", sf_2d|sf_priority_9|sf_vol_7, []),
 ("quest_completed", sf_2d|sf_priority_9|sf_vol_8, ["quest_completed.ogg"]),
 ("quest_succeeded", sf_2d|sf_priority_9|sf_vol_6, ["quest_succeeded.ogg"]),
 ("quest_concluded", sf_2d|sf_priority_9|sf_vol_7, ["quest_concluded.ogg"]),
 ("quest_failed", sf_2d|sf_priority_9|sf_vol_7, ["quest_failed.ogg"]),
 ("quest_cancelled", sf_2d|sf_priority_9|sf_vol_7, ["quest_cancelled.ogg"]),
 ("rain",sf_2d|sf_priority_10|sf_vol_4|sf_looping, ["rain_1.ogg"]),
 ("money_received",sf_2d|sf_priority_10|sf_vol_4, ["coins_dropped_1.ogg"]),
 ("money_paid",sf_2d|sf_priority_10|sf_vol_10, ["coins_dropped_2.ogg"]),
 ("sword_clash_1", sf_priority_5|sf_vol_8,["sword_clank_metal_09.ogg","sword_clank_metal_09b.ogg","sword_clank_metal_10.ogg","sword_clank_metal_10b.ogg","sword_clank_metal_12.ogg","sword_clank_metal_12b.ogg","sword_clank_metal_13.ogg","sword_clank_metal_13b.ogg"]),
 ("sword_clash_2", sf_priority_5|sf_vol_9,["s_swordClash2.wav"]),
 ("sword_clash_3", sf_priority_5|sf_vol_9,["s_swordClash3.wav"]),
 ("sword_swing", sf_vol_8|sf_priority_2,["s_swordSwing.wav"]),
 ("footstep_grass", sf_vol_4|sf_priority_1,["footstep_1.ogg","footstep_2.ogg","footstep_3.ogg","footstep_4.ogg"]),
 ("footstep_wood", sf_vol_5|sf_priority_1,["footstep_wood_1.ogg","footstep_wood_2.ogg","footstep_wood_4.ogg"]),
# ("footstep_wood", sf_vol_3|sf_priority_1,["footstep_stone_1.ogg","footstep_stone_3.ogg","footstep_stone_4.ogg"]),
 ("footstep_water", sf_vol_4|sf_priority_3,["water_walk_1.ogg","water_walk_2.ogg","water_walk_3.ogg","water_walk_4.ogg"]),
 ("footstep_horse",sf_priority_3|sf_vol_8, ["footstep_horse_5.ogg","footstep_horse_2.ogg","footstep_horse_3.ogg","footstep_horse_4.ogg"]),
# ("footstep_horse",0, ["s_footstep_horse_4b.wav","s_footstep_horse_4f.wav","s_footstep_horse_5b.wav","s_footstep_horse_5f.wav"]),
 ("footstep_horse_1b",sf_priority_3|sf_vol_8, ["s_footstep_horse_4b.wav","s_footstep_horse_4f.wav","s_footstep_horse_5b.wav","s_footstep_horse_5f.wav"]),
 ("footstep_horse_1f",sf_priority_3|sf_vol_8, ["s_footstep_horse_2b.wav","s_footstep_horse_2f.wav","s_footstep_horse_3b.wav","s_footstep_horse_3f.wav"]),
# ("footstep_horse_1f",sf_priority_3|sf_vol_8, ["footstep_horse_5.ogg","footstep_horse_2.ogg","footstep_horse_3.ogg","footstep_horse_4.ogg"]),
 ("footstep_horse_2b",sf_priority_3|sf_vol_8, ["s_footstep_horse_2b.wav"]),
 ("footstep_horse_2f",sf_priority_3|sf_vol_8, ["s_footstep_horse_2f.wav"]),
 ("footstep_horse_3b",sf_priority_3|sf_vol_8, ["s_footstep_horse_3b.wav"]),
 ("footstep_horse_3f",sf_priority_3|sf_vol_8, ["s_footstep_horse_3f.wav"]),
 ("footstep_horse_4b",sf_priority_3|sf_vol_8, ["s_footstep_horse_4b.wav"]),
 ("footstep_horse_4f",sf_priority_3|sf_vol_8, ["s_footstep_horse_4f.wav"]),
 ("footstep_horse_5b",sf_priority_3|sf_vol_8, ["s_footstep_horse_5b.wav"]),
 ("footstep_horse_5f",sf_priority_3|sf_vol_8, ["s_footstep_horse_5f.wav"]),
 ("jump_begin", sf_vol_6|sf_priority_9,["jump_begin.ogg"]),
 ("jump_end", sf_vol_5|sf_priority_9,["jump_end.ogg"]),
 ("jump_begin_water", sf_vol_3|sf_priority_9,["jump_begin_water.ogg"]),
 ("jump_end_water", sf_vol_3|sf_priority_9,["jump_end_water.ogg"]),
 ("horse_jump_begin", sf_vol_6|sf_priority_9,["horse_jump_begin.ogg"]),
 ("horse_jump_end", sf_vol_7|sf_priority_9,["horse_jump_end.ogg"]),
 ("horse_jump_begin_water", sf_vol_6|sf_priority_9,["jump_begin_water.ogg"]),
 ("horse_jump_end_water", sf_vol_6|sf_priority_9,["jump_end_water.ogg"]),

 ("release_bow",sf_vol_4, ["release_bow_1.ogg"]),
 ("release_crossbow",sf_vol_7, ["release_crossbow_1.ogg"]),
 ("throw_javelin",sf_vol_5, ["throw_javelin_2.ogg"]),
 ("throw_axe",sf_vol_7, ["throw_axe_1.ogg"]),
 ("throw_knife",sf_vol_5, ["throw_knife_1.ogg"]),
 ("throw_stone",sf_vol_7, ["throw_stone_1.ogg"]),

 ("reload_crossbow",sf_vol_3, ["reload_crossbow_1.ogg"]),
 ("reload_crossbow_continue",sf_vol_6, ["put_back_dagger.ogg"]),
 ("pull_bow",sf_vol_5, ["pull_bow_1.ogg"]),
 ("pull_arrow",sf_vol_5, ["pull_arrow.ogg"]),

 ("arrow_pass_by",sf_priority_7, ["arrow_pass_by_1.ogg","arrow_pass_by_2.ogg","arrow_pass_by_3.ogg","arrow_pass_by_4.ogg"]),
 ("bolt_pass_by",sf_priority_7, ["bolt_pass_by_1.ogg"]),
 ("javelin_pass_by",sf_priority_7, ["javelin_pass_by_1.ogg","javelin_pass_by_2.ogg"]),
 ("stone_pass_by",sf_vol_9|sf_priority_7, ["stone_pass_by_1.ogg"]),
 ("axe_pass_by",sf_priority_7, ["axe_pass_by_1.ogg"]),
 ("knife_pass_by",sf_priority_7, ["knife_pass_by_1.ogg"]),
 ("bullet_pass_by",sf_priority_7, ["arrow_whoosh_1.ogg"]),
 
 ("incoming_arrow_hit_ground",sf_priority_7|sf_vol_7, ["arrow_hit_ground_2.ogg","arrow_hit_ground_3.ogg","incoming_bullet_hit_ground_1.ogg"]),
 ("incoming_bolt_hit_ground",sf_priority_7|sf_vol_7, ["arrow_hit_ground_2.ogg","arrow_hit_ground_3.ogg","incoming_bullet_hit_ground_1.ogg"]),
 ("incoming_javelin_hit_ground",sf_priority_7|sf_vol_7, ["incoming_javelin_hit_ground_1.ogg"]),
 ("incoming_stone_hit_ground",sf_priority_7|sf_vol_7, ["incoming_stone_hit_ground_1.ogg"]),
 ("incoming_axe_hit_ground",sf_priority_7|sf_vol_7, ["incoming_javelin_hit_ground_1.ogg"]),
 ("incoming_knife_hit_ground",sf_priority_7|sf_vol_7, ["incoming_stone_hit_ground_1.ogg"]),
 ("incoming_bullet_hit_ground",sf_priority_7|sf_vol_7, ["incoming_bullet_hit_ground_1.ogg"]),

 ("outgoing_arrow_hit_ground",sf_priority_7|sf_vol_7, ["outgoing_arrow_hit_ground.ogg"]),
 ("outgoing_bolt_hit_ground",sf_priority_7|sf_vol_7,  ["outgoing_arrow_hit_ground.ogg"]),
 ("outgoing_javelin_hit_ground",sf_priority_7|sf_vol_10, ["outgoing_arrow_hit_ground.ogg"]),
 ("outgoing_stone_hit_ground",sf_priority_7|sf_vol_7, ["incoming_stone_hit_ground_1.ogg"]),
 ("outgoing_axe_hit_ground",sf_priority_7|sf_vol_7, ["incoming_javelin_hit_ground_1.ogg"]),
 ("outgoing_knife_hit_ground",sf_priority_7|sf_vol_7, ["incoming_stone_hit_ground_1.ogg"]),
 ("outgoing_bullet_hit_ground",sf_priority_7|sf_vol_7, ["incoming_bullet_hit_ground_1.ogg"]),


 ("draw_sword",sf_priority_2|sf_vol_4, ["draw_sword.ogg"]),
 ("put_back_sword",sf_priority_1|sf_vol_4, ["put_back_sword.ogg"]),
 ("draw_greatsword",sf_priority_2|sf_vol_4, ["draw_greatsword.ogg"]),
 ("put_back_greatsword",sf_priority_1|sf_vol_4, ["put_back_sword.ogg"]),
 ("draw_axe",sf_priority_2|sf_vol_4, ["draw_mace.ogg"]),
 ("put_back_axe",sf_priority_1|sf_vol_4, ["put_back_to_holster.ogg"]),
 ("draw_greataxe",sf_priority_2|sf_vol_4, ["draw_greataxe.ogg"]),
 ("put_back_greataxe",sf_priority_1|sf_vol_4, ["put_back_to_leather.ogg"]),
 ("draw_spear",sf_priority_2|sf_vol_4, ["draw_spear.ogg"]),
 ("put_back_spear",sf_priority_1|sf_vol_4, ["put_back_to_leather.ogg"]),
 ("draw_crossbow",sf_priority_2|sf_vol_4, ["draw_crossbow.ogg"]),
 ("put_back_crossbow",sf_priority_1|sf_vol_4, ["put_back_to_leather.ogg"]),
 ("draw_revolver",sf_priority_2|sf_vol_4, ["draw_from_holster.ogg"]),
 ("put_back_revolver",sf_priority_1|sf_vol_4, ["put_back_to_holster.ogg"]),
 ("draw_dagger",sf_priority_2|sf_vol_4, ["draw_dagger.ogg"]),
 ("put_back_dagger",sf_priority_1|sf_vol_4, ["put_back_dagger.ogg"]),
 ("draw_bow",sf_priority_2|sf_vol_4, ["draw_bow.ogg"]),
 ("put_back_bow",sf_priority_1|sf_vol_4, ["put_back_to_holster.ogg"]),
 ("draw_shield",sf_priority_2|sf_vol_4, ["draw_shield.ogg"]),
 ("put_back_shield",sf_priority_1|sf_vol_4, ["put_back_shield.ogg"]),
 ("draw_other",sf_priority_2|sf_vol_4, ["draw_other.ogg"]),
 ("put_back_other",sf_priority_1|sf_vol_4, ["draw_other2.ogg"]),

 ("body_fall_small",sf_priority_5|sf_vol_9, ["body_fall_small_1.ogg","body_fall_small_2.ogg"]),
 ("body_fall_big",sf_priority_6|sf_vol_8, ["body_fall_1.ogg","body_fall_2.ogg","body_fall_3.ogg"]),
# ("body_fall_very_big",sf_priority_9|sf_vol_10, ["body_fall_very_big_1.ogg"]),
 ("horse_body_fall_begin",sf_priority_6|sf_vol_10, ["horse_body_fall_begin_1.ogg"]),
 ("horse_body_fall_end",sf_priority_6|sf_vol_10, ["horse_body_fall_end_1.ogg","body_fall_2.ogg","body_fall_very_big_1.ogg"]),
 
## ("clang_metal",sf_priority_9, ["clang_metal_1.ogg","clang_metal_2.ogg","s_swordClash1.wav","s_swordClash2.wav","s_swordClash3.wav"]),
 ("hit_wood_wood",sf_priority_7|sf_vol_12, ["hit_wood_wood_1.ogg","hit_wood_wood_2.ogg","hit_wood_wood_3.ogg","hit_wood_wood_4.ogg","hit_wood_metal_4.ogg","hit_wood_metal_5.ogg","hit_wood_metal_6.ogg"]),#dummy
 ("hit_metal_metal",sf_priority_7|sf_vol_10, ["hit_metal_metal_3.ogg","hit_metal_metal_4.ogg",
                                              "hit_metal_metal_5.ogg","hit_metal_metal_6.ogg","hit_metal_metal_7.ogg","hit_metal_metal_8.ogg",
                                              "hit_metal_metal_9.ogg","hit_metal_metal_10.ogg",
                                              "clang_metal_1.ogg","clang_metal_2.ogg"]),
 ("hit_wood_metal",sf_priority_7|sf_vol_10, ["hit_metal_metal_1.ogg","hit_metal_metal_2.ogg","hit_wood_metal_7.ogg"]),
# ("clang_metal", sf_priority_9,["sword_clank_metal_09.ogg","sword_clank_metal_10.ogg","sword_clank_metal_12.ogg","sword_clank_metal_13.ogg"]),
## ("shield_hit_cut",sf_priority_5, ["shield_hit_cut_3.ogg","shield_hit_cut_4.ogg","shield_hit_cut_5.ogg"]),
 ("shield_hit_wood_wood",sf_priority_7|sf_vol_9, ["shield_hit_wood_wood_1.ogg","shield_hit_wood_wood_2.ogg","shield_hit_wood_wood_3.ogg"]),
 ("shield_hit_metal_metal",sf_priority_7|sf_vol_10, ["shield_hit_metal_metal_1.ogg","shield_hit_metal_metal_2.ogg","shield_hit_metal_metal_3.ogg","shield_hit_metal_metal_4.ogg"]),
 ("shield_hit_wood_metal",sf_priority_7|sf_vol_10, ["shield_hit_cut_3.ogg","shield_hit_cut_4.ogg","shield_hit_cut_5.ogg","shield_hit_cut_10.ogg"]), #(shield is wood)
 ("shield_hit_metal_wood",sf_priority_7|sf_vol_10, ["shield_hit_metal_wood_1.ogg","shield_hit_metal_wood_2.ogg","shield_hit_metal_wood_3.ogg"]),#(shield is metal)
 ("shield_broken",sf_priority_9, ["shield_broken.ogg"]),
 ("man_hit",sf_priority_7|sf_vol_7, ["man_hit_5.ogg","man_hit_6.ogg","man_hit_7.ogg","man_hit_8.ogg","man_hit_9.ogg","man_hit_10.ogg","man_hit_11.ogg","man_hit_12.ogg","man_hit_13.ogg","man_hit_14.ogg","man_hit_15.ogg",
                                      "man_hit_17.ogg","man_hit_18.ogg","man_hit_19.ogg","man_hit_22.ogg","man_hit_29.ogg","man_hit_32.ogg","man_hit_47.ogg","man_hit_57.ogg","man_hit_59.ogg"]),
 ("man_die",sf_priority_10|sf_vol_8,  ["man_death_1.ogg","man_death_8.ogg","man_death_8b.ogg","man_death_11.ogg","man_death_14.ogg","man_death_16.ogg","man_death_18.ogg","man_death_21.ogg","man_death_29.ogg","man_death_40.ogg","man_death_44.ogg","man_death_46.ogg","man_death_48.ogg","man_death_64.ogg"]),# ["man_fall_1.ogg","man_fall_2.ogg","man_fall_3.ogg","man_fall_4.ogg"]),
 ("woman_hit",sf_priority_7, ["woman_hit_2.ogg","woman_hit_3.ogg",
                              "woman_hit_b_2.ogg","woman_hit_b_4.ogg","woman_hit_b_6.ogg","woman_hit_b_7.ogg","woman_hit_b_8.ogg",
                              "woman_hit_b_11.ogg","woman_hit_b_14.ogg","woman_hit_b_16.ogg"]),
 ("woman_die",sf_priority_10|sf_vol_9, ["woman_fall_1.ogg","woman_hit_b_5.ogg"]),
 ("woman_yell",sf_priority_8|sf_vol_9, ["woman_yell_1.ogg","woman_yell_2.ogg"]),
 ("hide",0, ["s_hide.wav"]),
 ("unhide",0, ["s_unhide.wav"]),
 ("neigh",0, ["horse_exterior_whinny_01.ogg","horse_exterior_whinny_02.ogg","horse_exterior_whinny_03.ogg","horse_exterior_whinny_04.ogg","horse_exterior_whinny_05.ogg","horse_whinny.ogg"]),
 ("gallop",sf_vol_4, ["horse_gallop_3.ogg","horse_gallop_4.ogg","horse_gallop_5.ogg"]),
 ("battle",sf_vol_4, ["battle.ogg"]),
# ("bow_shoot_player",sf_priority_10|sf_vol_10, ["bow_shoot_4.ogg"]),
# ("bow_shoot",sf_priority_4, ["bow_shoot_4.ogg"]),
# ("crossbow_shoot",sf_priority_4, ["bow_shoot_2.ogg"]),
 ("arrow_hit_body",sf_priority_4, ["arrow_hit_body_1.ogg","arrow_hit_body_2.ogg","arrow_hit_body_3.ogg"]),
 ("metal_hit_low_armor_low_damage",sf_priority_5|sf_vol_9, ["sword_hit_lo_armor_lo_dmg_1.ogg","sword_hit_lo_armor_lo_dmg_2.ogg","sword_hit_lo_armor_lo_dmg_3.ogg"]),
 ("metal_hit_low_armor_high_damage",sf_priority_5|sf_vol_9, ["sword_hit_lo_armor_hi_dmg_1.ogg","sword_hit_lo_armor_hi_dmg_2.ogg","sword_hit_lo_armor_hi_dmg_3.ogg"]),
 ("metal_hit_high_armor_low_damage",sf_priority_5|sf_vol_9, ["metal_hit_high_armor_low_damage.ogg","metal_hit_high_armor_low_damage_2.ogg","metal_hit_high_armor_low_damage_3.ogg"]),
 ("metal_hit_high_armor_high_damage",sf_priority_5|sf_vol_9, ["sword_hit_hi_armor_hi_dmg_1.ogg","sword_hit_hi_armor_hi_dmg_2.ogg","sword_hit_hi_armor_hi_dmg_3.ogg"]),
 ("wooden_hit_low_armor_low_damage",sf_priority_5|sf_vol_9, ["blunt_hit_low_1.ogg","blunt_hit_low_2.ogg","blunt_hit_low_3.ogg"]),
 ("wooden_hit_low_armor_high_damage",sf_priority_5|sf_vol_9, ["blunt_hit_high_1.ogg","blunt_hit_high_2.ogg","blunt_hit_high_3.ogg"]),
 ("wooden_hit_high_armor_low_damage",sf_priority_5|sf_vol_9, ["wooden_hit_high_armor_low_damage.ogg","wooden_hit_high_armor_low_damage_2.ogg"]),
 ("wooden_hit_high_armor_high_damage",sf_priority_5|sf_vol_9, ["blunt_hit_high_1.ogg","blunt_hit_high_2.ogg","blunt_hit_high_3.ogg"]),
 ("mp_arrow_hit_target",sf_2d|sf_priority_15|sf_vol_9, ["mp_arrow_hit_target.ogg"]),
 ("blunt_hit",sf_priority_5|sf_vol_9, ["punch_1.ogg","punch_4.ogg","punch_4.ogg","punch_5.ogg"]),
 ("player_hit_by_arrow",sf_priority_10|sf_vol_10, ["player_hit_by_arrow.ogg"]),
 ("pistol_shot",sf_priority_10|sf_vol_10, ["fl_pistol.wav"]),
 ("man_grunt",sf_priority_6|sf_vol_4, ["man_excercise_1.ogg","man_excercise_2.ogg","man_excercise_4.ogg"]),
 ("man_breath_hard",sf_priority_3|sf_vol_8, ["man_ugh_1.ogg","man_ugh_2.ogg","man_ugh_4.ogg","man_ugh_7.ogg","man_ugh_12.ogg","man_ugh_13.ogg","man_ugh_17.ogg"]),
 ("man_stun",sf_priority_3|sf_vol_8, ["man_stun_1.ogg"]),
 ("man_grunt_long",sf_priority_6|sf_vol_7, ["man_grunt_1.ogg","man_grunt_2.ogg","man_grunt_3.ogg","man_grunt_5.ogg","man_grunt_13.ogg","man_grunt_14.ogg"]),
 ("man_yell",sf_priority_5|sf_vol_8, ["man_yell_4.ogg","man_yell_4_2.ogg","man_yell_7.ogg","man_yell_9.ogg","man_yell_11.ogg","man_yell_13.ogg","man_yell_15.ogg","man_yell_16.ogg","man_yell_17.ogg","man_yell_20.ogg","man_shortyell_4.ogg","man_shortyell_5.ogg","man_shortyell_6.ogg","man_shortyell_9.ogg","man_shortyell_11.ogg","man_shortyell_11b.ogg",
                        "man_yell_b_18.ogg","man_yell_22.ogg", "man_yell_c_20.ogg"]),
## not adequate, removed: "man_yell_b_21.ogg","man_yell_b_22.ogg","man_yell_b_23.ogg"]),
#TODONOW:
 ("man_warcry",sf_priority_5, ["man_insult_2.ogg","man_insult_3.ogg","man_insult_7.ogg","man_insult_9.ogg","man_insult_13.ogg","man_insult_15.ogg","man_insult_16.ogg"]),

 ("encounter_looters",sf_2d|sf_vol_8, ["encounter_river_pirates_5.ogg","encounter_river_pirates_6.ogg","encounter_river_pirates_9.ogg","encounter_river_pirates_10.ogg","encounter_river_pirates_4.ogg"]),

 ("encounter_bandits",sf_2d|sf_vol_8, ["encounter_bandit_2.ogg","encounter_bandit_9.ogg","encounter_bandit_12.ogg","encounter_bandit_13.ogg","encounter_bandit_15.ogg","encounter_bandit_16.ogg","encounter_bandit_10.ogg",]),
 ("encounter_farmers",sf_2d|sf_vol_8, ["encounter_farmer_2.ogg","encounter_farmer_5.ogg","encounter_farmer_7.ogg","encounter_farmer_9.ogg"]),
 ("encounter_sea_raiders",sf_2d|sf_vol_8, ["encounter_sea_raider_5.ogg","encounter_sea_raider_9.ogg","encounter_sea_raider_9b.ogg","encounter_sea_raider_10.ogg"]),
 ("encounter_steppe_bandits",sf_2d|sf_vol_8, ["encounter_steppe_bandit_3.ogg","encounter_steppe_bandit_3b.ogg","encounter_steppe_bandit_8.ogg","encounter_steppe_bandit_10.ogg","encounter_steppe_bandit_12.ogg"]),
 ("encounter_nobleman",sf_2d|sf_vol_8, ["encounter_nobleman_1.ogg"]),
 ("encounter_vaegirs_ally",sf_2d|sf_vol_8, ["encounter_vaegirs_ally.ogg","encounter_vaegirs_ally_2.ogg"]),
 ("encounter_vaegirs_neutral",sf_2d|sf_vol_8, ["encounter_vaegirs_neutral.ogg","encounter_vaegirs_neutral_2.ogg","encounter_vaegirs_neutral_3.ogg","encounter_vaegirs_neutral_4.ogg"]),
 ("encounter_vaegirs_enemy",sf_2d|sf_vol_8, ["encounter_vaegirs_neutral.ogg","encounter_vaegirs_neutral_2.ogg","encounter_vaegirs_neutral_3.ogg","encounter_vaegirs_neutral_4.ogg"]),
 ("sneak_town_halt",sf_2d, ["sneak_halt_1.ogg","sneak_halt_2.ogg"]),
 ("horse_walk",sf_priority_3|sf_vol_5, ["horse_walk_1.ogg","horse_walk_2.ogg","horse_walk_3.ogg","horse_walk_4.ogg"]),
 ("horse_trot",sf_priority_4|sf_vol_6, ["horse_trot_1.ogg","horse_trot_2.ogg","horse_trot_3.ogg","horse_trot_4.ogg"]),
 ("horse_canter",sf_priority_4|sf_vol_7, ["horse_canter_1.ogg","horse_canter_2.ogg","horse_canter_3.ogg","horse_canter_4.ogg"]),
 ("horse_gallop",sf_priority_5|sf_vol_8, ["horse_gallop_6.ogg","horse_gallop_7.ogg","horse_gallop_8.ogg","horse_gallop_9.ogg"]),
 ("horse_breath",sf_priority_1|sf_vol_4, ["horse_breath_4.ogg","horse_breath_5.ogg","horse_breath_6.ogg","horse_breath_7.ogg"]),
 ("horse_snort",sf_priority_1|sf_vol_2, ["horse_snort_1.ogg","horse_snort_2.ogg","horse_snort_3.ogg","horse_snort_4.ogg","horse_snort_5.ogg"]),
 ("horse_low_whinny",sf_vol_12, ["horse_whinny-1.ogg","horse_whinny-2.ogg"]),
 ("block_fist",0, ["block_fist_3.ogg","block_fist_4.ogg"]),
 ("man_hit_blunt_weak",sf_priority_5|sf_vol_10, ["man_hit_13.ogg","man_hit_29.ogg","man_hit_32.ogg","man_hit_47.ogg","man_hit_57.ogg"]),
 ("man_hit_blunt_strong",sf_priority_5|sf_vol_10, ["man_hit_13.ogg","man_hit_29.ogg","man_hit_32.ogg","man_hit_47.ogg","man_hit_57.ogg"]),
 ("man_hit_pierce_weak",sf_priority_5|sf_vol_10, ["man_hit_13.ogg","man_hit_29.ogg","man_hit_32.ogg","man_hit_47.ogg","man_hit_57.ogg"]),
 ("man_hit_pierce_strong",sf_priority_5|sf_vol_10, ["man_hit_13.ogg","man_hit_29.ogg","man_hit_32.ogg","man_hit_47.ogg","man_hit_57.ogg"]),
 ("man_hit_cut_weak",sf_priority_5|sf_vol_10, ["man_hit_13.ogg","man_hit_29.ogg","man_hit_32.ogg","man_hit_47.ogg","man_hit_57.ogg"]),
 ("man_hit_cut_strong",sf_priority_5|sf_vol_10, ["man_hit_13.ogg","man_hit_29.ogg","man_hit_32.ogg","man_hit_47.ogg","man_hit_57.ogg"]),
 ("man_victory",sf_priority_5|sf_vol_10, ["man_victory_3.ogg","man_victory_4.ogg","man_victory_5.ogg","man_victory_8.ogg","man_victory_15.ogg","man_victory_49.ogg","man_victory_52.ogg","man_victory_54.ogg","man_victory_57.ogg","man_victory_71.ogg"]),
 ("fire_loop",sf_priority_9|sf_vol_4|sf_looping|sf_start_at_random_pos, ["Fire_Torch_Loop3.ogg"]),
 ("torch_loop",sf_priority_9|sf_vol_4|sf_looping|sf_start_at_random_pos, ["Fire_Torch_Loop3.ogg"]),
 ("dummy_hit",sf_priority_9, ["shield_hit_cut_3.ogg","shield_hit_cut_5.ogg"]),
 ("dummy_destroyed",sf_priority_9, ["shield_broken.ogg"]),
 ("gourd_destroyed",sf_priority_9, ["shield_broken.ogg"]),#TODO
 ("cow_moo", sf_2d|sf_priority_9|sf_vol_8, ["cow_moo_1.ogg"]),
 ("cow_slaughter", sf_2d|sf_priority_9|sf_vol_8, ["cow_slaughter.ogg"]),
 ("distant_dog_bark", sf_2d|sf_priority_3|sf_vol_8, ["d_dog1.ogg","d_dog2.ogg","d_dog3.ogg","d_dog7.ogg"]),
 ("distant_owl", sf_2d|sf_priority_3|sf_vol_9, ["d_owl2.ogg","d_owl3.ogg","d_owl4.ogg"]),
 ("distant_chicken", sf_2d|sf_priority_3|sf_vol_8, ["d_chicken1.ogg","d_chicken2.ogg"]),
 ("distant_carpenter", sf_2d|sf_priority_3|sf_vol_3, ["d_carpenter1.ogg","d_saw_short3.ogg"]),
 ("distant_blacksmith", sf_2d|sf_priority_3|sf_vol_4, ["d_blacksmith2.ogg"]),
 ("arena_ambiance", sf_2d|sf_priority_8|sf_vol_3|sf_looping, ["arena_loop11.ogg"]),
 ("town_ambiance", sf_2d|sf_priority_8|sf_vol_3|sf_looping, ["town_loop_3.ogg"]),
 ("tutorial_fail", sf_2d|sf_vol_7,["cue_failure.ogg"]),
 ("your_flag_taken", sf_2d|sf_priority_10|sf_vol_10, ["your_flag_taken.ogg"]),
 ("enemy_flag_taken", sf_2d|sf_priority_10|sf_vol_10, ["enemy_flag_taken.ogg"]),
 ("flag_returned", sf_2d|sf_priority_10|sf_vol_10, ["your_flag_returned.ogg"]),
 ("team_scored_a_point", sf_2d|sf_priority_10|sf_vol_10, ["you_scored_a_point.ogg"]),
 ("enemy_scored_a_point", sf_2d|sf_priority_10|sf_vol_10, ["enemy_scored_a_point.ogg"]),
]



## 用于编写一些ai方面的功能
from smart_module.module_system.header_operations import *
from smart_module.module_system.header_parties import *
from smart_module.module_system.module_constants import *

Blank={
    "name":"AiBaseScripts",
    "enable":True,
    "version": "v2.0.0",
    "desc":"这里是描述信息",
    "actions":[
        ("Append@scripts",[
            ## 代码功能
        ]),
        ("Prepend@scripts",[]),
        ("Delete@scripts",[]),
        ("Replace@scripts",[
            ## 集合中只能有一个元素，否则会报错（替换只能精准替换，不能全部替换，而且，只能使用一个数据替换）
        ])
    ],
    ## 汉化
    "internationals":{
            "cns":{
                "dialogs":[

                ]
            }
    }
}


from smart_module.smart_core.smart_modules_log import *

################################################
###
### slot编号 操作
### 用于编译时自动获得slot编号,避免手动指定编号以后，与其它功能相互冲突
### 1.设置自动编号的开始编号
### 2.根据不同类型（troop,party,faction等等）自动生成编号
### 3.相同的slotName会获得相同的编号
### 4.slot编号最大为【2^20】 超过以后数据会丢失
###
################################################

class SmartModuleSlotManager:
    def __init__(self):
        self.config = {
            "troop":{
                "start":165,
            },
            "party":{
                "start":370,
            },
            "faction":{
                "start":160,
            },
            "scene":{
                "start":15,
            },
            "party_template":{
                "start":10,
            },
            "agent":{
                "start":30,
            },
            "quest":{
                "start":30,
            },
            "item":{
                "start":65,
            },
            "player":{
                "start":40,
            },
            "team":{
                "start":15,
            },
            "scene_prop":{
                "start":10,
                "current":10,
                "slots":{

                }
            }
        }

    def getSlotNo(self,type, slotName):
        typeConfig = self.config.get(type)
        ## 获得slot配置字典
        slots = typeConfig.get("slots")
        ## 如果字典为空时，就初始化
        if slots == None:
            typeConfig["slots"] = {}
            slots = typeConfig["slots"]

        ## 根据slot名称查找编号
        currentSlotNum = slots.get(slotName)

        ## 如果查找不到就生成一个编号
        if currentSlotNum == None:
            ## 获得当前已经使用的编号
            currentSlotNum = typeConfig.get("current")
            ## 如果当前使用的标号为空就使用start设置好的开始编号
            if currentSlotNum == None:
                currentSlotNum = typeConfig.get("start")
            ## 生成编号
            currentSlotNum = currentSlotNum + 1
            debug("【slot管理器】-为[{}]类型的slot[{}]名称生成新的编号：{}".format(type,slotName, currentSlotNum))
            ## 将生成的编号保存起来
            typeConfig["current"] = currentSlotNum
            ## 将生成的编号全部保存起来，避免相同名称重复生成
            slots[slotName] = currentSlotNum
        else:
            debug("【slot管理器】-已经存在[{}]类型的slot[{}]名称的编号：{}".format(type,slotName,currentSlotNum))
        ## 返回计算后的编号
        return currentSlotNum

    def getTroopSlotNo(self,slotName):
        return self.getSlotNo("troop", slotName)

    def getPartySlotNo(self,slotName):
        return self.getSlotNo("party", slotName)

    def getFactionSlotNo(self,slotName):
        return self.getSlotNo("faction", slotName)

    def getSceneSlotNo(self,slotName):
        return self.getSlotNo("scene", slotName)

    def getPartyTemplateSlotNo(self,slotName):
        return self.getSlotNo("party_template", slotName)

    def getAgentSlotNo(self,slotName):
        return self.getSlotNo("agent", slotName)

    def getQuestSlotNo(self,slotName):
        return self.getSlotNo("quest", slotName)

    def getItemSlotNo(self,slotName):
        return self.getSlotNo("item", slotName)

    def getPlayerSlotNo(self,slotName):
        return self.getSlotNo("player", slotName)

    def getTeamSlotNo(self,slotName):
        return self.getSlotNo("team", slotName)

    def getScenePropSlotNo(self,slotName):
        return self.getSlotNo("scene_prop", slotName)


## 必须保证唯一性，否则【编号会生成重复】的
smartModuleSlotManager = SmartModuleSlotManager()


if __name__ == '__main__':
    smartModuleSlotManager.getItemSlotNo("aaaaa")
    smartModuleSlotManager.getItemSlotNo("bbbbb")
    smartModuleSlotManager.getTroopSlotNo("ccccc")
    smartModuleSlotManager.getItemSlotNo("aaaaa")






from smart_module.module_system.module_parties import *
from smart_module.module_system.process_common import *
from smart_module.module_system.process_operations import *
from smart_module.module_system.process_tags_unused import tag_uses


def save_parties(parties,export_dir):
  file = open(export_dir + "parties.txt","w",encoding="utf-8")
  file.write("partiesfile version 1\n")
  file.write("%d %d\n"%(len(parties), len(parties)))
  for i_party in range(len(parties)):
    party = parties[i_party]
    if (party[5] >= 0):
      add_tag_use(tag_uses,tag_faction,party[5])

    file.write(" 1 %d %d "%(i_party, i_party))
#    file.write(" 1 %d "%(i_party))
    file.write("p_%s %s %d "%(convert_to_identifier(party[0]),replace_spaces(party[1]),party[2]))
    menu_no = 0
    menu_param = party[3]
    if (type(menu_param) == str):
      menu_no = find_object("mnu",menu_param)
      if (menu_no < 0):
        print( "Error: Unable to find menu-id :" + menu_param)
    else:
      menu_no = menu_param
    file.write("%d "%(menu_no))
    
    file.write("%d %d %d %d %d "%(party[4], party[5], party[6], party[6],party[7]))
    ai_behavior_object = 0
    ai_param = party[8]
    if (type(ai_param) == str):
      ai_behavior_object = find_object("p",ai_param)
      if (ai_behavior_object < 0):
        print( "Error: Unable to find party-id :" + ai_param)
    else:
      ai_behavior_object = ai_param
    file.write("%d %d "%(ai_behavior_object,ai_behavior_object))
    position = party[9]
    default_behavior_location = position
    file.write("%f %f "%(default_behavior_location[0],default_behavior_location[1]))
    file.write("%f %f "%(default_behavior_location[0],default_behavior_location[1]))
    file.write("%f %f 0.0 "%position)
    member_list = party[10]
    file.write("%d "%len(member_list))
    for member in member_list:
      add_tag_use(tag_uses,tag_troop,member[0])
      file.write("%d %d 0 %d "%(member[0],member[1],member[2]))
    bearing = 0.0
    if (len(party) > 11):
      bearing = (3.1415926 / 180.0) * party[11]
    file.write("\n%f\n"%(bearing))
  file.close()

def save_python_header(parties,src_dir):
  file = open(src_dir + "ID_parties.py","w",encoding="utf-8")
  for i_party in range(len(parties)):
    file.write("p_%s = %d\n"%(convert_to_identifier(parties[i_party][0]),i_party))
  file.close()


def processParties(context):
  print( "Exporting parties")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  tag_uses = load_tag_uses(export_dir)
  save_python_header(parties,src_dir)
  save_parties(parties,export_dir)
  save_tag_uses(export_dir, tag_uses)

  #print( "Generating C header...")
  #save_c_header()
  #print( "Generating Python header...")
  #print( "Finished.")



file = open("module_scripts.py", "r",encoding="utf-8")
lines = file.readlines()
file.close()

file = open("module_scripts.py", "w",encoding="utf-8")

level = 0
for line in lines:
  line = line.strip()
  acceptableindex = line.find("#")
  if (acceptableindex == -1):
      acceptableindex = len(line)
  level -= line.count("try_end", 0, acceptableindex)
  level -= line.count("end_try", 0, acceptableindex)
  level -= line.count("else_try", 0, acceptableindex)
  newlevel = level
  level_positive_change = 0
  newlevel += line.count("else_try", 0, acceptableindex)
  newlevel += line.count("(", 0, acceptableindex)
  newlevel += line.count("[", 0, acceptableindex)
  newlevel += line.count("try_begin", 0, acceptableindex)
  newlevel += line.count("try_for", 0, acceptableindex)
  level_positive_change = newlevel - level
  newlevel -= line.count(")", 0, acceptableindex)
  newlevel -= line.count("]", 0, acceptableindex)
  if (level_positive_change == 0):
    level = newlevel
  for i in range(level):
    file.write("  ")
  level = newlevel
  file.write("%s\n"%line)
file.close()



from smart_module.module_system.module_scene_props import *

from smart_module.module_system.process_operations import *


def save_scene_props(variable_list,variable_uses,tag_uses,quick_strings,export_dir):
  ofile = open(export_dir + "scene_props.txt","w",encoding="utf-8")
  ofile.write("scene_propsfile version 1\n")
  ofile.write(" %d\n"%(len(scene_props)))
  for scene_prop in scene_props:
    ofile.write("spr_%s %d %d %s %s "%(scene_prop[0], scene_prop[1], get_spr_hit_points(scene_prop[1]), scene_prop[2], scene_prop[3]))
    save_simple_triggers(ofile,scene_prop[4]  , variable_list,variable_uses,tag_uses,quick_strings)
    ofile.write("\n")
  ofile.close()


def save_python_header(src_dir):
  file = open(src_dir + "ID_scene_props.py","w",encoding="utf-8")
  for i_scene_prop in range(len(scene_props)):
    file.write("spr_%s = %d\n"%(scene_props[i_scene_prop][0],i_scene_prop))
  file.close()


def processSceneProps(context):
  print( "Exporting scene props...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  variable_uses = []
  variables = load_variables(export_dir,variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_scene_props(variables,variable_uses,tag_uses,quick_strings,export_dir)
  save_variables(export_dir,variables,variable_uses)
  save_tag_uses(export_dir,tag_uses)
  save_quick_strings(export_dir,quick_strings)



from smart_module.module_system.module_map_icons import *

from smart_module.module_system.process_operations import *


def save_map_icons(variable_list,variable_uses,tag_uses,quick_strings,export_dir):
  ofile = open(export_dir + "map_icons.txt","w",encoding="utf-8")
  ofile.write("map_icons_file version 1\n")
  ofile.write("%d\n"%len(map_icons))
  for map_icon in map_icons:
    triggers = []
    if (len(map_icon) >= 8):
      ofile.write("%s %d %s %f %d %f %f %f "%(map_icon[0],map_icon[1],map_icon[2],map_icon[3],map_icon[4],map_icon[5],map_icon[6],map_icon[7]))
      if (len(map_icon) == 9):
        triggers = map_icon[8]
    else:
      ofile.write("%s %d %s %f %d 0 0 0 "%(map_icon[0],map_icon[1],map_icon[2],map_icon[3],map_icon[4]))
      if (len(map_icon) == 6):
        triggers = map_icon[5]
    save_simple_triggers(ofile,triggers, variable_list,variable_uses,tag_uses,quick_strings)
    ofile.write("\n")
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_map_icons.py","w",encoding="utf-8")
  for i_map_icon in range(len(map_icons)):
    ofile.write("icon_%s = %d\n"%(map_icons[i_map_icon][0],i_map_icon))
  ofile.close()



def processMapIcons(context):
  print( "Exporting map icons...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  variable_uses = []
  variables = load_variables(export_dir,variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_map_icons(variables,variable_uses,tag_uses,quick_strings,export_dir)
  save_variables(export_dir,variables,variable_uses)
  save_tag_uses(export_dir,tag_uses)
  save_quick_strings(export_dir,quick_strings)



from smart_module.module_system.header_quests import *

####################################################################################################################
#  Each quest record contains the following fields:
#  1) Quest id: used for referencing quests in other files. The prefix qst_ is automatically added before each quest-id.
#  2) Quest Name: Name displayed in the quest screen.
#  3) Quest flags. See header_quests.py for a list of available flags
#  4) Quest Description: Description displayed in the quest screen.
#
# Note that you may call the opcode setup_quest_text for setting up the name and description
####################################################################################################################

quests = [
# Note : This is defined as the first governer quest in module_constants.py: 
 ("deliver_message", "Deliver Message to {s13}", qf_random_quest,
  "{!}{s9} asked you to take a message to {s13}. {s13} was at {s4} when you were given this quest."
  ),
 ("deliver_message_to_enemy_lord", "Deliver Message to {s13}", qf_random_quest,
  "{!}{s9} asked you to take a message to {s13} of {s15}. {s13} was at {s4} when you were given this quest."
  ),
 ("raise_troops", "Raise {reg1} {s14}", qf_random_quest,
  "{!}{s9} asked you to raise {reg1} {s14} and bring them to him."
  ),
 ("escort_lady", "Escort {s13} to {s14}", qf_random_quest,
  "{!}None"
  ),
## ("rescue_lady_under_siege", "Rescue {s3} from {s4}", qf_random_quest,
##  "{s1} asked you to rescue his {s7} {s3} from {s4} and return her back to him."
##  ),
## ("deliver_message_to_lover", "Deliver Message to {s3}", qf_random_quest,
##  "{s1} asked you to take a message to his lover {s3} at {s4}."
##  ),
## ("bring_prisoners_to_enemy", "Bring Prisoners to {s4}", qf_random_quest,
##  "{s1} asked you to bring {reg1} {s3} as prisoners to the guards at {s4}."
##  ),
## ("bring_reinforcements_to_siege", "Bring Reinforcements to the Siege of {s5}", qf_random_quest,
##  "{s1} asked you to bring {reg1} {s3} to {s4} at the siege of {s5}."
##  ),
## ("deliver_supply_to_center_under_siege", "Deliver Supplies to {s5}", qf_random_quest,
##  "TODO: Take {reg1} cartloads of supplies from constable {s3} and deliver them to constable {s4} at {s5}."
##  ),
 ("deal_with_bandits_at_lords_village", "Save the Village of {s15} from Marauding Bandits", qf_random_quest,
  "{!}{s13} asked you to deal with the bandits who took refuge in his village of {s15} and then report back to him."
  ),
 ("collect_taxes", "Collect Taxes from {s3}", qf_random_quest,
  "{!}{s9} asked you to collect taxes from {s3}. He offered to leave you one-fifth of all the money you collect there."
  ),
 ("hunt_down_fugitive", "Hunt Down {s4}", qf_random_quest,
  "{!}{s9} asked you to hunt down the fugitive named {s4}. He is currently believed to be at {s3}."
  ),
## ("capture_messenger", "Capture {s3}", qf_random_quest,
##  "{s1} asked you to capture a {s3} and bring him back."
##  ),
## ("bring_back_deserters", "Bring {reg1} {s3}", qf_random_quest,
##  "{s1} asked you to bring {reg1} {s3}."
##  ),
 ("kill_local_merchant", "Assassinate Local Merchant at {s3}", qf_random_quest,
  "{!}{s9} asked you to assassinate a local merchant at {s3}."
  ),
 ("bring_back_runaway_serfs", "Bring Back Runaway Serfs", qf_random_quest,
  "{!}{s9} asked you to bring back the three groups of runaway serfs back to {s2}. He said all three groups must be running away in the direction of {s3}."
  ),
 ("follow_spy", "Follow the Spy to Meeting", qf_random_quest,
  "{!}{s11} asked you to follow the spy that will leave {s12}. You must be careful not to be seen by the spy during his travel, or else he may get suspicious and turn back. Once the spy meets with his accomplice, you are to ambush and capture them and bring them both back to {s11}."
  ),
 ("capture_enemy_hero", "Capture a Lord from {s13}", qf_random_quest,
  "{!}TODO: {s11} asked you to capture a lord from {s13}."
  ),
 ("lend_companion", "Lend Your Companion {s3} to {s9}", qf_random_quest,
  "{!}{s9} asked you to lend your companion {s3} to him for a week."
  ),
 ("collect_debt", "Collect the Debt {s3} Owes to {s9}", qf_random_quest,
  "{!}{s9} asked you to collect the debt of {reg4} denars {s3} owes to him."
  ),
## ("capture_conspirators", "Capture Conspirators", qf_random_quest,
##  "TODO: {s1} asked you to capture all troops in {reg1} conspirator parties that plan to rebel against him and join {s3}."
##  ),
## ("defend_nobles_against_peasants", "Defend Nobles Against Peasants", qf_random_quest,
##  "TODO: {s1} asked you to defend {reg1} noble parties against peasants."l
##  ),
 ("incriminate_loyal_commander", "Incriminate the Loyal Commander of {s13}, {s16}", qf_random_quest,
  "{!}None"
  ),
# ("raid_caravan_to_start_war", "Raid {reg13} Caravans of {s13}", qf_random_quest,   #This is now a dynamic quest, integrated into the provocation system
#  "None"
#  ),
 ("meet_spy_in_enemy_town", "Meet Spy in {s13}", qf_random_quest,
  "{!}None"
  ),
 ("capture_prisoners", "Bring {reg1} {s3} Prisoners", qf_random_quest,
  "{!}{s9} wanted you to bring him {reg1} {s3} as prisoners."
  ),

## ("hunt_down_raiders", "Hunt Down Raiders",qf_random_quest,
##  "{s1} asked you to hunt down and punish the raiders that attacked a village near {s3} before they reach the safety of their base at {s4}."
##  ),

##################
# Enemy Kingdom Lord quests
##################
# Note : This is defined as the first enemy lord quest in module_constants.py:
 ("lend_surgeon", "Lend Your Surgeon {s3} to {s1}", qf_random_quest,
  "{!}Lend your experienced surgeon {s3} to {s1}."
  ),

##################
# Kingdom Army quests
##################
# Note : This is defined as lord quests end in module_constants.py:
 ("follow_army", "Follow {s9}'s Army", qf_random_quest,
  "{!}None"
  ),
 ("report_to_army", "Report to {s13}, the Marshall", qf_random_quest,
  "{!}None"
  ),
# Note : This is defined as the first army quest in module_constants.py:
# maybe disable these army quests, except as volunteer quests that add to the capacity of the army
 ("deliver_cattle_to_army", "Deliver {reg3} Heads of Cattle to {s13}", qf_random_quest,
  "{!}None"
  ),
 ("join_siege_with_army", "Join the Siege of {s14}", qf_random_quest,
  "{!}None"
  ),
 ("screen_army", "Screen the Advance of {s13}'s Army", qf_random_quest,
  "{!}None"
  ),
 ("scout_waypoints", "Scout {s13}, {s14} and {s15}", qf_random_quest,
  "{!}None"
  ),


##################
# Kingdom Lady quests
##################
# Note : This is defined as the first kingdom lady quest in module_constants.py:
#Rescue lord by replace will become a 
 ("rescue_lord_by_replace", "Rescue {s13} from {s14}", qf_random_quest,
  "{!}None"
  ),
 ("deliver_message_to_prisoner_lord", "Deliver Message to {s13} at {s14}", qf_random_quest,
  "{!}None"
  ),

#Courtship quests
  ("duel_for_lady", "Challenge {s13} to a Trial of Arms", qf_random_quest,
  "{!}None"
  ),

  ("duel_courtship_rival", "Challenge {s13} to a Trial of Arms (optional)", qf_random_quest,
  "{!}None"
  ),

#Other duel quests
  ("duel_avenge_insult", "Challenge {s13} to a Trial of Arms", qf_random_quest,
  "{!}None"
  ),
  
  
  
##################
# Mayor quests
##################
# Note : This is defined as the first mayor quest in module_constants.py: 
 ("move_cattle_herd", "Move Cattle Herd to {s13}", qf_random_quest,
  "{!}Guildmaster of {s10} asked you to move a cattle herd to {s13}."
  ),
 ("escort_merchant_caravan", "Escort Merchant Caravan to {s8}", qf_random_quest, #make this a non-random quest?
  "{!}Escort the merchant caravan to the town of {s8}."
  ),
 ("deliver_wine", "Deliver {reg5} Units of {s6} to {s4}", qf_random_quest,
  "{!}{s9} of {s3} asked you to deliver {reg5} units of {s6} to the tavern in {s4} in 7 days."
  ),
 ("troublesome_bandits", "Hunt Down Troublesome Bandits", qf_random_quest,
  "{!}{s9} of {s4} asked you to hunt down the troublesome bandits in the vicinity of the town."
  ),
  
 ("kidnapped_girl", "Ransom Girl from Bandits", qf_random_quest,
  "{!}Guildmaster of {s4} gave you {reg12} denars to pay the ransom of a girl kidnapped by bandits.\
 You are to meet the bandits near {s3} and pay them the ransom fee.\
 After that you are to bring the girl back to {s4}."
  ),
  
 ("persuade_lords_to_make_peace", "Make Sure Two Lords Do Not Object to Peace", qf_random_quest, #possibly deprecate., or change effects
  "{!}Guildmaster of {s4} promised you {reg12} denars if you can make sure that\
 {s12} and {s13} no longer pose a threat to a peace settlement between {s15} and {s14}.\
 In order to do that, you must either convince them or make sure they fall captive and remain so until a peace agreement is made."
  ),
  
 ("deal_with_looters", "Deal with Looters", qf_random_quest,
  "{!}The Guildmaster of {s4} has asked you to deal with several bands of looters around {s4}, and bring back any goods you recover."
  ),
 ("deal_with_night_bandits", "Deal with Night Bandits", qf_random_quest,
  "{!}TODO: The Guildmaster of {s14} has asked you to deal with night bandits at {s14}."
  ),

############
# Village Elder quests
############
# Note : This is defined as the first village elder quest in module_constants.py:
 ("deliver_grain", "Bring wheat to {s3}", qf_random_quest,
  "{!}The elder of the village of {s3} asked you to bring them {reg5} packs of wheat.."
  ), 
 ("deliver_cattle", "Deliver {reg5} Heads of Cattle to {s3}", qf_random_quest,
  "{!}The elder of the village of {s3} asked you to bring {reg5} heads of cattle."
  ), 
 ("train_peasants_against_bandits", "Train the Peasants of {s13} Against Bandits.", qf_random_quest,
  "{!}None"
  ), 
# Deliver horses, Deliver food, Escort_Caravan, Hunt bandits, Ransom Merchant.
## ("capture_nobleman", "Capture Nobleman",qf_random_quest,
##  "{s1} wanted you to capture an enemy nobleman on his way from {s3} to {s4}. He said the nobleman would leave {s3} in {reg1} days."
##  ),

# Bandit quests: Capture rich merchant, capture banker, kill manhunters?..

# Note : This is defined as the last village elder quest in module_constants.py:
 ("eliminate_bandits_infesting_village", "Save the Village of {s7} from Marauding Bandits", qf_random_quest,
  "{!}A villager from {s7} begged you to save their village from the bandits that took refuge there."
  ),


 # Tutorial quest
## ("destroy_dummies", "Destroy Dummies", qf_show_progression,
##  "Trainer ordered you to destroy 10 dummies in the training camp."
##     ),

  #Courtship and marriage quests begin here
  ("visit_lady", "Visit Lady", qf_random_quest,
  "{!}None"
  ),
  ("formal_marriage_proposal", "Formal Marriage Proposal", qf_random_quest,
  "{!}None"
  ),  #Make a formal proposal to a bride's father or brother
  ("obtain_liege_blessing", "Formal Marriage Proposal", qf_random_quest,
  "{!}None"
  ),  #The equivalent of the above -- ask permission of a groom's liege. Is currently not used
  ("wed_betrothed", "Wed Your Betrothed", qf_random_quest,
  "{!}None"
  ),  #in this case, the giver troop is the father or guardian of the bride, object troop is the bride
  ("wed_betrothed_female", "Wed Your Betrothed", qf_random_quest,
  "{!}None"
  ),  #in this case, the giver troop is the spouse

  
 # Join Kingdom quest
  ("join_faction", "Give Oath of Homage to {s1}", qf_random_quest,
  "{!}Find {s1} and give him your oath of homage."
  ),

 # Rebel against Kingdom quest
 ("rebel_against_kingdom", "Help {s13} Claim the Throne of {s14}", qf_random_quest,
  "{!}None"
  ),

  #Political quests begin here
 ("consult_with_minister", "Consult With Minister", qf_random_quest, "{!}Consult your minister, {s11}, currently at {s12}"),
 
 ("organize_feast",        "Organize Feast", qf_random_quest,        "{!}Bring goods for a feast to your spouse {s11}, currently at {s12}"),
 ("resolve_dispute",       "Resolve Dispute", qf_random_quest,       "{!}Resolve the dispute between {s11} and {s12}"),
 ("offer_gift",            "Procure Gift", qf_random_quest,          "{!}Give {s10} a gift to provide to {reg4?her:his} {s11}, {s12}"),
 ("denounce_lord",         "Denounce Lord", qf_random_quest,         "{!}Denounce {s11} in Public"),
 ("intrigue_against_lord", "Intrigue against Lord", qf_random_quest, "{!}Criticize {s11} in Private"),
 
 
  #Dynamic quests begin here
  #These quests are determined dynamically by external conditions -- bandits who have carried out a raid, an impending war, etc...
 ("track_down_bandits", "Track Down Bandits", qf_random_quest,
  "{!}{s9} of {s4} asked you to track down {s6}, who attacked travellers on the roads near town."
  ), #this is a fairly simple quest for the early game to make the town guildmaster's description of the economy a little more relevant, and also to give the player a reason to talk to other neutral parties on the map
   
 ("track_down_provocateurs", "Track Down Provocateurs", qf_random_quest,
  "{!}{s9} of {s4} asked you to track down a group of thugs, hired to create a border incident between {s5} and {s6}."
  ), 
 ("retaliate_for_border_incident", "Retaliate for a Border Incident", qf_random_quest,
  "{!}{s9} of {s4} asked you to defeat {s5} of the {s7} in battle, defusing tension in the {s8} to go to war."
  ), #perhaps replaces persuade_lords_to_make_peace
  
 ("raid_caravan_to_start_war", "Attack a Neutral Caravan to Provoke War", qf_random_quest,
  "{!}placeholder",
  ), 

  ("cause_provocation", "Give a Kingdom Provocation to Attack Another", qf_random_quest,
  "{!}placeholder",
  ), #replaces raid_caravan_to_start_war
  
 ("rescue_prisoner", "Rescue or Ransom a Prisoner", qf_random_quest,
  "{!}placeholder"
  ), #possibly replaces rescue lord

 ("destroy_bandit_lair", "Destroy Bandit Lair", qf_random_quest,
  "{!}{s9} of {s4} asked you to discover a {s6} and destroy it."
  ), 
  
 ("blank_quest_2", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),
  
 ("blank_quest_3", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_4", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_5", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_6", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_7", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_8", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_9", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_10", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_11", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_12", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_13", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_14", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_15", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_16", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_17", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_18", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_19", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_20", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_21", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_22", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_23", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_24", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_25", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_26", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("blank_quest_27", "{!}blank_quest", qf_random_quest,
  "{!}placeholder"
  ),

 ("collect_men", "Collect Five Men", 0,
  "{!}{s9} asked you to collect at least 5 men before you move against the bandits threatening the townsmen. You can recruit soldiers from villages as well as town taverns. You can find {s9} at the tavern in {s4} when you have think you have enough men."
  ), 
  
  ("learn_where_merchant_brother_is", "Learn Where the Hostages are Held.", 0,
  "{!}placeholder."
  ), 
  
  ("save_relative_of_merchant", "Attack the Bandit Lair", 0,
  "{!}placeholder."
  ),   

  ("save_town_from_bandits", "Save Town from Bandits", 0,
  "{!}placeholder."
  ),   
  
 ("quests_end", "Quests End", 0, "{!}."),
]



from smart_module.module_system.module_info import *
from smart_module.module_system.module_info_pages import *

def save_info_pages(export_dir):
  ofile = open(export_dir + "info_pages.txt","w",encoding="utf-8")
  ofile.write("infopagesfile version 1\n")
  ofile.write("%d\n"%(len(info_pages)))
  for i_info_page in range(len(info_pages)):
    info_page = info_pages[i_info_page]
    ofile.write("ip_%s %s %s"%(info_page[0],info_page[1].replace(" ","_"), info_page[2].replace(" ","_")))
    ofile.write("ip_%s %s %s"%(info_page[0],info_page[1].replace(" ","_"), info_page[2].replace(" ","_")))
    ofile.write("\n")
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_info_pages.py","w",encoding="utf-8")
  for i_info_page in range(len(info_pages)):
    ofile.write("ip_%s = %d\n"%(info_pages[i_info_page][0],i_info_page))
  ofile.write("\n\n")
  ofile.close()


def processInfoPages(context):
  print( "Exporting info_page data...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_info_pages(export_dir)
  save_python_header(src_dir)
  




################################################
###
### 处理器：元命令，第二部分
### 系统内置处理器
###
################################################
from smart_module.module_system.module_game_menus import game_menus, jump_to_menu
from smart_module.smart_core.base.exception import SmartModuleExceptin
from smart_module.smart_core.base.meta_command.smart_modules_selector import SelectorMatcher, SelectedResult


class AppendProcessor:
    '''
        【追加处理器】:将数据添加到选择数据行的[后边]，四大处理器之一
    '''
    def __init__(self):
        pass

    def __str__(self):
        return "执行器：AppendProcessor"

    def process(self,selectorResult:SelectedResult,customDatas:list):
        '''
            将数据处理进去

            1.获得目标对象
            2.获得目标对象的父级对象
            3.根据下标将数据插入到指定位置

        :param selectorResult:
        :return: 无返回值
        '''
        if type(customDatas) != list:
            raise SmartModuleExceptin("提供给【追加】处理器的数据必须是list结构，请检查：" + selector)

        resultItems = selectorResult.getTargetResultItemList()

        if len(resultItems) > 1:
            raise SmartModuleExceptin("【追加】处理器只能处理单个结果集合（选择器命中多个数据），请检查：" + selector)
        if len(resultItems) != 1:
            raise SmartModuleExceptin("【追加】处理器只能处理单个结果集合（选择器没有命中任何数据），请检查：" + selector)
        resultItem = resultItems[0]

        parentHitIndex = resultItem.getHitIndex()
        parentData = resultItem.getHitParentData()

        customDataSize = len(customDatas)
        for i in range(customDataSize):
            index = customDataSize - i - 1
            customData = customDatas[index]
            parentData.insert(parentHitIndex + 1, customData)


class PrependProcessor:
    '''
        【前置处理器】:将数据添加到选择数据行的[前边]，四大处理器之一
    '''
    def __init__(self):
        pass

    def __str__(self):
        return "执行器：PrependProcessor"

    def process(self,selectorResult:SelectedResult,customDatas:list):
        '''
            将数据处理进去

            1.获得目标对象
            2.获得目标对象的父级对象
            3.根据下标将数据插入到指定位置

        :param selectorResult:
        :return: 无返回值
        '''
        if type(customDatas) != list:
            raise SmartModuleExceptin("提供给【前置】处理器的数据必须是list结构，请检查：" + selectorResult.getSelector())

        resultItems = selectorResult.getTargetResultItemList()

        if len(resultItems) > 1:
            raise SmartModuleExceptin("【前置】处理器只能处理单个结果集合（选择器命中多个数据），请检查：" + selectorResult.getSelector())
        if len(resultItems) != 1:
            raise SmartModuleExceptin("【前置】处理器只能处理单个结果集合（选择器没有命中任何数据），请检查：" + selectorResult.getSelector())
        resultItem = resultItems[0]

        parentHitIndex = resultItem.getHitIndex()
        parentData = resultItem.getHitParentData()

        customDataSize = len(customDatas)
        for i in range(customDataSize):
            index = customDataSize - i - 1
            customData = customDatas[index]
            parentData.insert(parentHitIndex, customData)


class ReplaceProcessor:
    '''
        【替换处理器】:将数据添加到选择数据行的[替换]，四大处理器之一

        只支持替换一个数据,第二个数据会被抛弃
    '''
    def __init__(self):
        pass

    def __str__(self):
        return "执行器：ReplaceProcessor"

    def process(self,selectorResult:SelectedResult,customData):
        '''
            将数据处理进去

            1.获得目标对象
            2.获得目标对象的父级对象
            3.根据下标将数据插入到指定位置

        :param selectorResult:
        :return: 无返回值
        '''

        resultItems = selectorResult.getTargetResultItemList()

        if len(resultItems) > 1:
            raise SmartModuleExceptin("【替换】处理器只能处理单个结果集合（选择器命中多个数据），请检查：" + selectorResult.selector)
        if len(resultItems) != 1:
            raise SmartModuleExceptin("【替换】处理器只能处理单个结果集合（选择器没有命中任何数据），请检查：" + selectorResult.selector)
        if len(customData) > 1:
            raise SmartModuleExceptin("【替换】处理器只能处理单个结果集合（用户数据有多个），请检查，选择器：{}，提供数据：{}".format(selectorResult.selector,str(customData)))

        resultItem = resultItems[0]

        parentHitIndex = resultItem.getHitIndex()
        parentData = resultItem.getHitParentData()
        parentData[parentHitIndex] = customData[0]



class DeleteProcessor:
    '''
        【删除处理器】:将数据添加到选择数据行的[删除]，四大处理器之一
    '''
    def __init__(self):
        pass

    def __str__(self):
        return "执行器：DeleteProcessor"

    def process(self,selectorResult:SelectedResult,customData = None):
        '''
            将数据处理进去

            1.获得目标对象
            2.获得目标对象的父级对象
            3.根据下标将数据插入到指定位置

        :param selectorResult:
        :return: 无返回值
        '''
        resultItems = selectorResult.getTargetResultItemList()
        for resultItem in resultItems:
            hitData = resultItem.getHitData()
            parentData = resultItem.getHitParentData()
            parentData.remove(hitData)

if __name__ == '__main__':
    ## 选择器
    selector = SelectorMatcher()

    ## 前置处理器
    prependProcessor = PrependProcessor()
    selectedDataResult = selector.select(game_menus, "start_game_0>5>continue")
    prependProcessor.process(selectedDataResult, [
        ("test1", [], "Test...",[(jump_to_menu, "mnu_start_game_1"),]),
        ("test2", [], "Test...",[(jump_to_menu, "mnu_start_game_1"),]),
        ("test3", [], "Test...",[(jump_to_menu, "mnu_start_game_1"),]),
    ])
    selectedDataResult.printResultData()

    ## 追加处理器
    appendProcessor = AppendProcessor()
    selectedDataResult = selector.select(game_menus, "start_game_0>5>continue")
    appendProcessor.process(selectedDataResult, [
        ("test1", [], "Test...",[(jump_to_menu, "mnu_start_game_1"),]),
        ("test2", [], "Test...",[(jump_to_menu, "mnu_start_game_1"),]),
        ("test3", [], "Test...",[(jump_to_menu, "mnu_start_game_1"),]),
    ])
    selectedDataResult.printResultData()

    ## 替换处理器
    replaceProcessor = ReplaceProcessor()
    selectedDataResult = selector.select(game_menus, "start_game_0>5>go_back")
    replaceProcessor.process(selectedDataResult, ("test1_go", [], "Test...", [(jump_to_menu, "mnu_start_game_1"), ]), )
    selectedDataResult.printResultData()

    ## 删除处理器
    deleteProcessor = DeleteProcessor()
    selectedDataResult = selector.select(game_menus, "start_game_0>5>test1")
    deleteProcessor.process(selectedDataResult)
    selectedDataResult.printResultData()


imod_plain = 0
imod_cracked = 1
imod_rusty = 2
imod_bent = 3
imod_chipped = 4
imod_battered = 5
imod_poor = 6
imod_crude = 7
imod_old = 8
imod_cheap = 9
imod_fine = 10
imod_well_made = 11
imod_sharp = 12
imod_balanced = 13
imod_tempered = 14
imod_deadly = 15
imod_exquisite = 16
imod_masterwork = 17
imod_heavy = 18
imod_strong = 19
imod_powerful = 20
imod_tattered = 21
imod_ragged = 22
imod_rough = 23
imod_sturdy = 24
imod_thick = 25
imod_hardened = 26
imod_reinforced = 27
imod_superb = 28
imod_lordly = 29
imod_lame = 30
imod_swaybacked = 31
imod_stubborn = 32
imod_timid = 33
imod_meek = 34
imod_spirited = 35
imod_champion = 36
imod_fresh = 37
imod_day_old = 38
imod_two_day_old = 39
imod_smelling = 40
imod_rotten = 41
imod_large_bag = 42


imodbit_plain = 1
imodbit_cracked = 2
imodbit_rusty = 4
imodbit_bent = 8
imodbit_chipped = 16
imodbit_battered = 32
imodbit_poor = 64
imodbit_crude = 128
imodbit_old = 256
imodbit_cheap = 512
imodbit_fine = 1024
imodbit_well_made = 2048
imodbit_sharp = 4096
imodbit_balanced = 8192
imodbit_tempered = 16384
imodbit_deadly = 32768
imodbit_exquisite = 65536
imodbit_masterwork = 131072
imodbit_heavy = 262144
imodbit_strong = 524288
imodbit_powerful = 1048576
imodbit_tattered = 2097152
imodbit_ragged = 4194304
imodbit_rough = 8388608
imodbit_sturdy = 16777216
imodbit_thick = 33554432
imodbit_hardened = 67108864
imodbit_reinforced = 134217728
imodbit_superb = 268435456
imodbit_lordly = 536870912
imodbit_lame = 1073741824
imodbit_swaybacked = 2147483648
imodbit_stubborn = 4294967296
imodbit_timid = 8589934592
imodbit_meek = 17179869184
imodbit_spirited = 34359738368
imodbit_champion = 68719476736
imodbit_fresh = 137438953472
imodbit_day_old = 274877906944
imodbit_two_day_old = 549755813888
imodbit_smelling = 1099511627776
imodbit_rotten = 2199023255552
imodbit_large_bag = 4398046511104



from smart_module.module_system.header_common import *
from smart_module.module_system.header_skills import *

####################################################################################################################
#  Each skill contains the following fields:
#  1) Skill id (string): used for referencing skills in other files. The prefix skl_ is automatically added before each skill-id .
#  2) Skill name (string).
#  3) Skill flags (int). See header_skills.py for a list of available flags
#  4) Maximum level of the skill (int).
#  5) Skill description (string): used in character window for explaining the skills.
# 
####################################################################################################################

#Hardcoded skills are {names (indexes, beginning with 0)}:
# Trade (1)
# Leadership (2)
# Prisoner Management (3)
# First Aid (9)
# Surgery (10)
# Wound Treatment (11)
# Inventory Management (12)
# Spotting (13)
# Pathfinding (14)
# Tactics (15)
# Tracking (16)
# Trainer (17)
# Engineer (18)
# Horse Archery (24)
# Riding (25)
# Athletics (26)
# Shield (27)
# Weapon Master (28)
# Power Draw (34)
# Power Throw (35)
# Power Strike (36)
# Ironflesh (37)
#
# The effects of these skills can only be removed if the skill is disabled with sf_inactive flag.
# If you want to add a new skill, use the reserved skills or use non-hardcoded skills.

skills = [
  ("trade","Trade",sf_base_att_cha|sf_effects_party,10,"Every level of this skill reduces your trade penalty by 5%%. (Party skill)"),
  ("leadership","Leadership",sf_base_att_cha,10,"Every point increases maximum number of troops you can command by 5, increases your party morale and reduces troop wages by 5%%. (Leader skill)"),
  ("prisoner_management", "Prisoner Management",sf_base_att_cha,10,"Every level of this skill increases maximum number of prisoners by %d. (Leader skill)"), 
  ("reserved_1","Reserved Skill 1",sf_base_att_cha|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_2","Reserved Skill 2",sf_base_att_cha|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_3","Reserved Skill 3",sf_base_att_cha|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_4","Reserved Skill 4",sf_base_att_cha|sf_inactive,10,"This is a reserved skill."), 
  ("persuasion","Persuasion", sf_base_att_int,10, "This skill helps you make other people accept your point of view. It also lowers the minimum level of relationship needed to get NPCs to do what you want. (Personal skill)"),
  ("engineer","Engineer",sf_base_att_int|sf_effects_party,10,"This skill allows you to construct siege equipment and fief improvements more efficiently. (Party skill)"),
  ("first_aid", "First Aid",sf_base_att_int|sf_effects_party,10,"Heroes regain 5%% per skill level of hit-points lost during mission. (Party skill)"), 
  ("surgery","Surgery",sf_base_att_int|sf_effects_party,10,"Each point to this skill gives a 4%% chance that a mortally struck party member will be wounded rather than killed. (Party skill)"), 
  ("wound_treatment","Wound Treatment",sf_base_att_int|sf_effects_party,10,"Party healing speed is increased by 20%% per level of this skill. (Party skill)"), 
  ("inventory_management","Inventory Management",sf_base_att_int,10,"Increases inventory capacity by +6 per skill level. (Leader skill)"), 
  ("spotting","Spotting",sf_base_att_int|sf_effects_party,10,"Party seeing range is increased by 10%% per skill level. (Party skill)"),
  ("pathfinding","Path-finding",sf_base_att_int|sf_effects_party,10,"Party map speed is increased by 3%% per skill level. (Party skill)"), 
  ("tactics","Tactics",sf_base_att_int|sf_effects_party,10,"Every two levels of this skill increases starting battle advantage by 1. (Party skill)"),
  ("tracking","Tracking",sf_base_att_int|sf_effects_party,10,"Tracks become more informative. (Party skill)"),
  ("trainer","Trainer",sf_base_att_int,10,"Every day, each hero with this skill adds some experience to every other member of the party whose level is lower than his/hers. Experience gained goes as: {0,4,10,16,23,30,38,46,55,65,80}. (Personal skill)"),
  ("reserved_5","Reserved Skill 5",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_6","Reserved Skill 6",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_7","Reserved Skill 7",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_8","Reserved Skill 8",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("looting","Looting",sf_base_att_agi|sf_effects_party,10,"This skill increases the amount of loot obtained by 10%% per skill level. (Party skill)"), 
  ("horse_archery","Horse Archery",sf_base_att_agi,10,"Reduces damage and accuracy penalties for archery and throwing from horseback. (Personal skill)"),
  ("riding","Riding",sf_base_att_agi,10,"Enables you to ride horses of higher difficulty levels and increases your riding speed and manuever. (Personal skill)"),
  ("athletics","Athletics",sf_base_att_agi,10,"Improves your running speed. (Personal skill)"),
  ("shield","Shield",sf_base_att_agi,10,"Reduces damage to shields (by 8%% per skill level) and improves shield speed and coverage. (Personal skill)"),
  ("weapon_master","Weapon Master",sf_base_att_agi,10,"Makes it easier to learn weapon proficiencies and increases the proficiency limits. Limits go as: 60, 100, 140, 180, 220, 260, 300, 340, 380, 420. (Personal skill)"),
  ("reserved_9","Reserved Skill 9",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_10","Reserved Skill 10",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_11","Reserved Skill 11",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_12","Reserved Skill 12",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_13","Reserved Skill 13",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("power_draw","Power Draw",sf_base_att_str,10,"Lets character use more powerful bows. Each point to this skill (up to four plus power-draw requirement of the bow) increases bow damage by 14%%. (Personal skill)"),
  ("power_throw","Power Throw",sf_base_att_str,10,"Each point to this skill increases throwing damage by 10%%. (Personal skill)"),
  ("power_strike","Power Strike",sf_base_att_str,10,"Each point to this skill increases melee damage by 8%%. (Personal skill)"),
  ("ironflesh","Ironflesh",sf_base_att_str,10,"Each point to this skill increases hit points by +2. (Personal skill)"), 
  ("reserved_14","Reserved Skill 14",sf_base_att_str|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_15","Reserved Skill 15",sf_base_att_str|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_16","Reserved Skill 16",sf_base_att_str|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_17","Reserved Skill 17",sf_base_att_str|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_18","Reserved Skill 18",sf_base_att_str|sf_inactive,10,"This is a reserved skill."), 
]



from smart_module.module_system.ID_factions import fac_commoners
from smart_module.module_system.header_game_menus import *
from smart_module.module_system.header_operations import *
from smart_module.module_system.header_skills import *
from smart_module.module_system.header_terrain_types import *
from smart_module.module_system.header_troops import *
from smart_module.module_system.module_constants import *
from smart_module.module_system.module_troops import *
from smart_module.smart_core.utils.pythonUtils import mergeList
from smart_module.smart_modules.base.StringBaseScripts import skill_str_begin

config = {
    ## 收获时间，每4小时收集一次
    "harvestTime":4,
    "survivalTypes":[
        {
            "typeName":"collect_apple",
            "typeNameCn":"采 摘 苹 果",
            "terrains":[rt_forest,rt_mountain_forest,rt_steppe_forest],
            ## 奖励物品
            "reward":"itm_apples",
            ## 最大值100（百分比）
            "probability":30,
            ## 技能增强机率(至少一技能)
            "probabilitySkill": skl_spotting,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount":5,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName":"collect_date_fruit",
            "typeNameCn":"采 摘 枣 子",
            "terrains":[rt_desert_forest],
            ## 奖励物品
            "reward":"itm_raw_date_fruit",
            ## 最大值100（百分比）
            "probability":30,
            ## 技能增强机率(至少一技能)
            "probabilitySkill": skl_spotting,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount":5,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName": "hunt_pig",
            "typeNameCn":"打 猎 野 猪",
            "terrains":[rt_forest,rt_mountain_forest,rt_steppe_forest],
            ## 奖励物品
            "reward": "itm_pork",
            ## 最大值100（百分比）
            "probability": 30,
            ## 技能增强机率
            "probabilitySkill": skl_power_draw,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount": 5,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName": "hunt_fish",
            "typeNameCn":"捕 捞 河 鱼",
            "terrains":[rt_water,rt_river,rt_bridge,rt_snow_forest],
            ## 奖励物品
            "reward": "itm_smoked_fish",
            ## 最大值100（百分比）
            "probability": 30,
            ## 技能增强机率
            "probabilitySkill": skl_power_draw,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount": 5,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName": "fell_wooden_stick",
            "typeNameCn":"砍 伐 树 枝",
            "terrains":[rt_forest,rt_mountain_forest,rt_steppe_forest,rt_snow_forest,rt_desert_forest],
            ## 奖励物品
            "reward": "itm_wooden_stick",
            ## 最大值100（百分比）
            "probability": 30,
            ## 技能增强机率
            "probabilitySkill": skl_power_strike,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount": 20,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName": "mining_stone",
            "typeNameCn":"采 集 石 子",
            ## 限制地形
            "terrains":[rt_mountain,rt_mountain_forest,rt_desert],
            ## 奖励物品
            "reward": "itm_stones",
            ## 最大值100（百分比）
            "probability": 30,
            ## 技能增强机率
            "probabilitySkill": skl_engineer,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount": 20,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName": "collect_grain",
            "typeNameCn":"收 集 野 麦",
            ## 限制地形
            "terrains":[rt_plain],
            ## 奖励物品
            "reward": "itm_grain",
            ## 最大值100（百分比）
            "probability": 10,
            ## 技能增强机率
            "probabilitySkill": skl_spotting,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount": 3,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName": "collect_cabbages",
            "typeNameCn":"收 集 野 菜",
            ## 限制地形
            "terrains":[rt_plain],
            ## 奖励物品
            "reward": "itm_cabbages",
            ## 最大值100（百分比）
            "probability": 10,
            ## 技能增强机率
            "probabilitySkill": skl_spotting,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 5,
            ## 每次获得数量
            "amount": 3,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
        {
            "typeName": "hunt_hourse",
            "typeNameCn":"驯 服 野 马",
            "terrains":[rt_steppe,rt_steppe_forest],
            ## 奖励物品
            "reward": "itm_steppe_horse",
            ## 最大值100（百分比）
            "probability": 10,
            ## 技能增强机率
            "probabilitySkill": skl_riding,
            ## 每一个技能增强5%机率
            "probabilitySkillEnhance": 2,
            ## 每次获得数量
            "amount": 1,
            ## 技能增强数量
            "amountSkill": skl_looting,
            ## 每一个技能增强10%数量
            "amountSkillEnhance": 10,
            ## 描述信息
            "desc":"每 一 点 侦 察 技 能 增 加 5% 的 机 率 ， 每 一 点 掠 夺 技 能 增 加 10% 的 数 量"
        },
    ]
}



## 收获时间
harvestTime = config.get("harvestTime")
## 测试
# harvestTime = 1


## 初始化survivalTypes
## 1.生成数据保存列表
## 2.生成收集类型字符列表
## 3.生成汉化信息
def initSurvivalTypes():
    ## 汉化列表
    gameStringsList = []
    ## 菜单选项
    gameMenuOptionList = []
    ## 生存类型
    survivalTypes = config.get("survivalTypes")

    for survivalType in survivalTypes:
        ## 生存名称
        typeName = survivalType.get("typeName")
        typeNameCn = survivalType.get("typeNameCn")
        ## 字符串列表 mno_survival_collect
        optionName = "survival_{}".format(typeName)
        optionName_ref = "mno_survival_{}".format(typeName)
        ## 汉化列表
        gameStringsList.append("{}|{}".format(optionName_ref,typeNameCn))

        ## 限制地形
        terrains = survivalType.get("terrains")
        ## 奖励
        reward = survivalType.get("reward")
        ## 机率
        probability = survivalType.get("probability")
        ## 机率所需技能
        probabilitySkill = survivalType.get("probabilitySkill")
        ## 机率所需技能加成
        probabilitySkillEnhance = survivalType.get("probabilitySkillEnhance")
        ## 物品数量
        amount = survivalType.get("amount")
        ## 物品数量所需要技能
        amountSkill = survivalType.get("amountSkill")
        ## 物品数量所需要技能加成
        amountSkillEnhance = survivalType.get("amountSkillEnhance")

        ## 条件
        conditions = []
        conditions.append((party_get_current_terrain,":terrain","p_main_party"))
        for i in range(len(terrains) - 1):
            terrain = terrains[i]
            conditions.append((this_or_next|eq,":terrain",terrain))

        conditions.append((eq, ":terrain", terrains[len(terrains) - 1]))
        ## 结果
        results = []
        results.append((assign,"$g_survival_reward",reward))
        results.append((assign,"$g_survival_probability",probability))
        results.append((assign,"$g_survival_probabilitySkill",probabilitySkill))
        results.append((assign,"$g_survival_probabilitySkillEnhance",probabilitySkillEnhance))
        results.append((assign,"$g_survival_amount",amount))
        results.append((assign,"$g_survival_amountSkill",amountSkill))
        results.append((assign,"$g_survival_amountSkillEnhance",amountSkillEnhance))
        results.append((call_script,"script_start_work_at_wild"))

        gameMenuOptionList.append((optionName,conditions,typeName,results))
    ## 返回
    gameMenuOptionList.append(("go_back", [], "Go back",[(jump_to_menu,"mnu_camp")]))
    return {
        "gameMenuOptionList":gameMenuOptionList,
        "gameStringsList":gameStringsList,
    }


datas = initSurvivalTypes()

gameMenuOptionList = datas.get("gameMenuOptionList")
gameStringsList = datas.get("gameStringsList")

## $g_survival_enable

is_normal = 0 ## 正常
is_working = 1 ## 工作中

pis_workman = 3


WARN_COLOR = 0xfffc9f03


survivalInTheWild={
    "name":"SurvivalInTheWild",
    "enable":True,
    "version": "v2.0.0",
    "desc":'''
        野外生存
            采 摘 苹 果
            采 摘 枣 子
            打 猎 野 猪
            捕 捞 河 鱼
            砍 伐 树 枝
            采 集 石 子
            收 集 野 麦
            收 集 野 菜
            驯 服 野 马
    ''',
    "actions":[
        ("Append@scripts",[
            ("start_work_at_wild",[
                ##(assign,"$g_camp_mode", 1),
                ##(assign, "$g_infinite_camping", 0),
                ##(assign, "$g_player_icon_state", pis_ship),

                (store_skill_level,":probabilitySkillCount","$g_survival_probabilitySkill"),
                (str_store_troop_name,s0,"trp_player"),
                (try_begin),
                    (le,":probabilitySkillCount",0),
                    (store_mul,":strIndex","$g_survival_probabilitySkill",2),
                    (val_add,":strIndex",skill_str_begin),
                    (str_store_string,s1,":strIndex"),
                    (display_message,"@{s0} need skill {s1}",WARN_COLOR),
                (else_try),
                    (assign,"$g_player_icon_state",pis_workman),
                    (rest_for_hours_interactive, 100000, 5, 1), #rest while not attackable
                    (assign,"$g_survival_enable",is_working),
                    (change_screen_return),
                (try_end),
            ]),
        ]),
        ("Append@simple_triggers",[
            (harvestTime,[
                ## 已经开启了野外生存
                (eq,"$g_survival_enable",is_working),
                (store_skill_level,":probabilitySkillCount","$g_survival_probabilitySkill"),
                (str_store_troop_name,s0,"trp_player"),
                (store_mul, ":probabilitySkillEnhance", ":probabilitySkillCount", "$g_survival_probabilitySkillEnhance"),
                ## 获得总机率
                (store_add, ":change", "$g_survival_probability", ":probabilitySkillEnhance"),
                ## 机率最大为100%
                (val_clamp, ":change", 0, 100),
                (store_random_in_range, ":isGet", 0, 100),
                ## 是否成功获得
                (le, ":isGet", ":change"),
                (store_skill_level, ":amountSkillCount", "$g_survival_amountSkill"),
                (store_mul, ":amountSkillEnhance", ":amountSkillCount", "$g_survival_amountSkillEnhance"),
                ## 数量技能增加最100%
                (val_clamp, ":amountSkillEnhance", 0, 100),
                (store_mul, ":plusAmount", "$g_survival_amount", ":amountSkillEnhance"),
                (store_add, ":total", "$g_survival_amount", ":plusAmount"),
                (call_script, "script_get_troop_inventory_free_amount", "trp_player"),
                (assign, ":storageMax", reg0),
                (try_begin),
                    (eq, ":storageMax", 0),
                    (assign, "$g_survival_enable", is_normal),
                    (assign, "$g_player_icon_state", pis_normal),
                    (rest_for_hours, 0, 0, 0),
                    (display_message, "@{s0} storage is full",WARN_COLOR),
                (else_try),
                    (troop_add_items, "trp_player", "$g_survival_reward", ":total"),
                (try_end),
             ]),
            (0,[
                (eq,"$g_survival_enable",is_working),
                (map_free),
                (assign,"$g_survival_enable",is_normal),
                (assign, "$g_player_icon_state", pis_normal),
             ])
        ]),
        ("AddCampOption",[
            ("camp_action_survival_in_the_wild", [], "survival in the wild",
             [
                (jump_to_menu,"mnu_survival_wild"),
             ]
            ),
        ]),
        ("Append@game_menus",[
            ("survival_wild", mnf_disable_all_keys,
             "survival in the wild.",
             "none",
             [],
             gameMenuOptionList
             ),
        ]),
        ("Replace@simple_triggers|#60",[
            (0,[
                (troop_get_inventory_slot, ":cur_horse", "trp_player", 8), #horse slot
                (assign, ":new_icon", -1),
                (try_begin),
                  (eq, "$g_player_icon_state", pis_normal),
                  (try_begin),
                    (ge, ":cur_horse", 0),
                    (assign, ":new_icon", "icon_player_horseman"),
                  (else_try),
                    (assign, ":new_icon", "icon_player"),
                  (try_end),
                (else_try),
                  (eq, "$g_player_icon_state", pis_camping),
                  (assign, ":new_icon", "icon_camp"),
                (else_try),
                  (eq, "$g_player_icon_state", pis_ship),
                  (assign, ":new_icon", "icon_ship"),
                (else_try),
                  (eq, "$g_player_icon_state", pis_workman),
                  (assign, ":new_icon", "icon_axeman"),
                (try_end),
                (neq, ":new_icon", "$g_player_party_icon"),
                (assign, "$g_player_party_icon", ":new_icon"),
                (party_set_icon, "p_main_party", ":new_icon"),
             ])
        ])
    ],
    "internationals":{
            "cns":{
                "game_menus":mergeList(gameStringsList,[
                    "mno_camp_action_survival_in_the_wild|野 外 生 存。",
                    "menu_survival_wild|野 外 生 存 技 能"
                ]),
                "quick_strings":[
                    "qstr_{s0}_need_skill_{s1}|{s0}：我 还 不 会 【{s1}】 技 能",
                    "qstr_{s0}_storage_is_full|{s0}：物 品 栏 已 经 满 了。"
                ]
            }
    }
}


from smart_module.module_system.module_info import *
from smart_module.module_system.process_common import *
from smart_module.module_system.process_operations import *


def processGlobalVariablesUnused(context):
  print( "Checking global variable usages...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  variable_uses = []
  variables = load_variables(export_dir,variable_uses)
  i = 0
  while (i < len(variables)):
    if (variable_uses[i] == 0):
      print( "WARNING: Global variable never used: " + variables[i])
    i = i + 1



# -*- coding: utf-8 -*-
import re

from smart_module.module_system.module_game_menus import game_menus
from smart_module.smart_core.base.exception import SmartModuleExceptin

## 【选择器】正则表达式
from smart_module.smart_core.utils.pythonUtils import convertToIdentifierWithNoLowercase



################################################
###
### 【选择器】：元命令，第一部分
### 系统内置选择器功能   选择器结果和选择器功能
###
################################################


class SelectedResultItem:
    '''
        命中数据项：一个选择器可能会命中多个数据
    '''
    def __init__(self,selectedData,selectedIndex,subSelectors,parentDatas):
        self.selectedData = selectedData
        self.selectedIndex = selectedIndex
        self.subSelectors = subSelectors
        self.parentDatas = parentDatas

    def getHitData(self):
        '''
            数据有可能为空，需要做判断处理
        :return:
        '''
        return self.selectedData

    def getHitIndex(self):
        '''
            获得匹配对象在父级中的索引
        :return:
        '''
        return self.selectedIndex

    def getHitParentData(self):
        '''
            获得匹配对象父级数据
        :return:
        '''
        return self.parentDatas



class SelectedResult:
    '''
        选择结果对象：通过选择器检索后获得的对象，主要包含
        1.已经选择的数据：每一个子选择器都会检索出一个对象
        2.子选择器的数据：如果有子选择器时，会记录下一个子选择器的结果（用于链路）
        3.父选择器的数据：如果有子选择器时，会记录上一个子选择器的结果（用于链路）
        4.获得数据可以直接修改后改变

        age>18>name

        age为父选择器
        name为子选择器
    '''
    def __init__(self,selectedDatas:list,selector):
        '''
            selectedDatas为数组，每一个对象中包含三个值
            1.选择中的数据
            2.选择中的值的下标
            3.【选择中的数据（也就是1.项）】所在的数据集合（父数据）
        :param selectedDatas:
        '''
        self.selectedDatas = selectedDatas

        ## 命中数据的选择器（字符串）
        self.selector = selector
        self.childrenResultList = []

        ## 用于临时保存结果集
        self._targetResultList = []
        self._targetResultItemList = []

    def getSelector(self):
        return self.selector

    def isHit(self):
        '''
            是否检索到数据
        :return:
        '''
        return len(self.selectedDatas) > 0

    def isMultiResult(self):
        '''
            是否同时匹配到多个对象
        :return:
        '''
        return len(self.selectedDatas) > 1

    def getResultItems(self):
        return self.selectedDatas

    def printResultData(self):
        print(self)


    def addChildResult(self, childrenResult):
        '''
            有子选择器时会被设置
        :param childrenResult:
        :return:
        '''
        self.childrenResultList.append(childrenResult)

    def getChildResults(self):
        return self.childrenResultList

    def setParentResult(self, parentResult):
        '''
            有父选择器时会被设置
        :param parentResult:
        :return:
        '''
        self.parentResult = parentResult

    def getParentResult(self):
        return self.parentResult



    def getTargetResult(self):
        '''
            获得目录数据：最后一个子元素的结果
                1.如果没有子选择器，就是第一个选择的结果
                2.如果有多个子选择器，就是最后一个选择的结果
        :return:
        '''
        if hasattr(self,"childrenResult"):
            return self.getChildResult().getTargetResult()
        else:
            return self

    def getTargetResultList(self):
        '''
            获得所有命中数据的[SelectedResult]
            可以追溯任何一级，便于用于更多用途
        :return:
        '''
        self._targetResultList = []
        self._foreach_children_item_result(self)
        return self._targetResultList


    def _foreach_children_result(self,childrenResult):
        if len(childrenResult.childrenResultList):
            for childrenResult in childrenResult.childrenResultList:
                self._foreach_children_item_result(childrenResult)
        else:
            self._targetResultList.append(childrenResult)

    def getTargetResultItemList(self):
        '''
            获得所有命中数据的[SelectedResultItem]
            可以直接处理数据，但【不能追溯父级】
        :return:
        '''
        self._targetResultItemList = []
        self._foreach_children_item_result(self)
        return self._targetResultItemList

    def _foreach_children_item_result(self, childrenResult):
        if len(childrenResult.childrenResultList):
            for childrenResult in childrenResult.childrenResultList:
                self._foreach_children_item_result(childrenResult)
        else:
            for resultItem in childrenResult.getResultItems():
                self._targetResultItemList.append(resultItem)






class SelectorMatcher:
    '''
        【选择器匹配器】：主要用于
    '''
    def __init__(self):
        self._selectorRegex = "^((((\w+([\*%^$])*(&\d+)?)(:\w+([\*%^$])*(&\d+)?)*)|#(\d+|last|first)))(>\d+>(((\w+([\*%^$])*(&\d+)?)(:\w+([\*%^$])*(&\d+)?)*)|#(\d+|last|first)))*$"

        ## 【是否是子选择器】正则表达式
        self._childrenRegex = "^(.+)(>\d+>(.+))+$"

        ## 用于获得下一个子选择器
        self._firstChildrenRegex = "^((((\w+([\*%^$])*(&\d+)?)(:\w+([\*%^$])*(&\d+)?)*)|#(\d+|last|first)))>(\d+)>(.+)$"

        self._simpleRegex = "^(((\w+([\*%^$])*(&\d+)?)(:\w+([\*%^$])*(&\d+)?)*)|#(\d+|last|first))$"

        ## 【匹配符子选择器】正则表达式
        self._matchSelector = "^(\w+)([\*%^$]*)(&(\d+))?$"

        ## 【索引子选择器】正则表达式
        self._indexSelector = "^#(\d+|last|first)$"

    def _checkSelector(self, selector:str):
        '''
            用于检验选择器格式是否正确
            参考：文档链接: https://share.mubu.com/doc/2Vgf0jfvew1 密码: 22d6

            【选择器】：一个完整的选择器由至少一个子选择器组成，子选择器有两路类型，索引子选择器和匹配符子选择器
            索引子选择器：
                1.#数字，直接通过数据的索引来匹配数据，此种方式最方便，速度也最快，但是缺点是下标会随着数据增加而改变，不建议使用在一级选择器中，以下为建议使用场景
                    1.选择游戏开始脚本（如，script_game_start）
                    2.子元素中的定位（且子元素不会增加个数）
                    3.用于定位第一个元素，方便置顶等操作
            匹配符子选择器：由两部分组成
                1.通配符：由两个部分组成
                    1.匹配值：就是一个表达式，如，name,age18,for_name等等
                    2.附加后缀：对匹配值作特殊能力加强
                        1.*：匹配时只要包含【匹配值】就可以了，如：name*，包含name就行,匹配数据：name18,18name,aaanamebbb
                        2.^：匹配时需要以【匹配值】开头，如：age18^，以age18开头,匹配数据：age1819,age18name,age18_name
                        3.$：匹配时需要以【匹配值】结尾，如：for_name$，以for_name结尾，匹配数据：this_for_name,18_for_name,test_for_name
                2.字段限定符（可选）：一条数据有多个字段，每一个子选择器都只针对一个字段，默认和第0个字段作对比
                    1.&数字：用于限定【通配符】是在和数据中的第几个数据作对比

            多个子选择器之间有两种关系，平级（:）和子级(>数字>)的关系
                1.平级，用于多个条件确定一个数据：age18&0:name&1，含义：使用age18匹配一行数据的第0个位置，使用name匹配一行数据的第1个位置，两个条件都满足时，确定这一行数据
                2.子级，用于深入选择，：age18&1:>4>:name&2，含义：使用age18匹配一行数据的第0个位置，成功后,获取第4位的值作为父级，然后再使用name和父级的第1位置的子数据对比
        :param selector:
        :return:
        '''
        matcher = re.match(self._selectorRegex, selector)
        if not matcher:
            raise SmartModuleExceptin("选择器语法错误，请检查：" + selector)

    def _findDataBySubSelector(self, dataRows, subSelectors:str):
        '''
            处理没有子级的选择器
        :param dataRow:
        :param subSelectors:
        :return:
        '''
        if "#" in subSelectors:
            matcher = re.match(self._indexSelector, subSelectors)
            if not matcher:
                raise SmartModuleExceptin("索引子选择器格式错误，selector：" + subSelectors)
            index = matcher.group(1)
            ## 第一个元素
            if "first" == index:
                index = 0
            ## 最后一个元素
            elif "last" == index:
                index = len(dataRows) -1
            else:
                index = int(index)
            if(index >= len(dataRows)):
                raise SmartModuleExceptin("索引值超出数据长度，selector：" + subSelectors)
            ## 特殊处理：
            ## 防止父级别的数据为空
            return SelectedResult([SelectedResultItem(dataRows[index] if index >=0 else dataRows, index if len(dataRows) > 0 else 0,subSelectors, dataRows)],subSelectors)
        else:
            subSelectorList = subSelectors.split(":")
            list = []
            for index in range(len(dataRows)):
                dataRow = dataRows[index]
                if self._isMatchSelector(dataRow, subSelectorList):
                    list.append(SelectedResultItem(dataRow,index,subSelectors,dataRows))
            if len(list) > 0:
                return SelectedResult(list,subSelectors)
            return SelectedResult([SelectedResultItem(None, -1,subSelectors, dataRows)],subSelectors)

    def _isMatchSelector(self, dataRow, subSelectors:list):
        '''
            当前数据是否匹配指定的选择器集合（可能只有一个）
        :param dataRow: 一行数据
        :param subSelectors: 匹配符子选择器
        :return: 匹配成功时返回True,失败时返回False
        '''
        for selector in subSelectors:
            matcher = re.match(self._matchSelector, selector)
            if not matcher:
                raise SmartModuleExceptin("匹配符子选择器格式错误，selector：" + selector)
            express = matcher.group(1)
            subfixs = matcher.group(2).strip()
            ## 如果没有指定索引，就默认使用第0位
            fieldIndex = int(matcher.group(4)) if matcher.group(4) != None else 0
            if(len(dataRow) <= fieldIndex):
                raise SmartModuleExceptin("数据行错误,或者选择器错误,数据行：【{}】，选择器：【{}】".format(str(dataRow),selector))
            field = dataRow[fieldIndex]
            if type(field) != str and len(subfixs) > 1:
                raise SmartModuleExceptin("附加后缀只能应用在字符串格式的字段上，selector：" + selector)
            ## 格式化处理
            if "%" in subfixs:
                field = convertToIdentifierWithNoLowercase(field)

            isMatch = False
            if "*" in subfixs:
                isMatch = field.find(express) > 0
            elif "^" in subfixs:
                isMatch = field.startswith(express)
            elif "$" in subfixs:
                isMatch = field.endswith(express)
            else:
                isMatch = field == express
            if isMatch == False:
                return False
        return True


    def select(self, dataRows, selector:str) -> SelectedResult:
        '''
            通过选择器检索数据,主要用于选择器的主要功能


        :param dataRow:
        :param selector:
        :return: 通过选择器检索的结果（SelectedResult类型）
        '''

        ## 子选择器
        isChildrenMatcher = re.match(self._firstChildrenRegex, selector)
        if isChildrenMatcher:
            subSelectors = isChildrenMatcher.group(1)
            childrenFieldIndex = int(isChildrenMatcher.group(11))
            remainSelectors = isChildrenMatcher.group(12)
            parentResultData = self._findDataBySubSelector(dataRows, subSelectors)
            if parentResultData.isHit():
                resultItems = parentResultData.getResultItems()
                for resultItem in resultItems:
                    parentData = resultItem.getHitData()
                    if parentData == None:
                        raise SmartModuleExceptin("没有匹配任何数据，请仔细检查选择器：【{}】".format(subSelectors))
                    if type(parentData) != list and type(parentData) != tuple:
                        raise SmartModuleExceptin("结果不是一个list或tuple无法进行再次选择，selector：【{}】 parentData:{}".format(subSelectors,parentData))
                    if childrenFieldIndex >= len(parentData):
                        raise SmartModuleExceptin("子选择器索引值超出父数据长度，selector：{} parentData:{}".format(selector,parentData))
                    childrenData = parentData[childrenFieldIndex]
                    ## 查询子数据
                    childrenResultData = self.select(childrenData, remainSelectors)
                    ## 设置上级数据
                    childrenResultData.setParentResult(parentResultData)
                    ## 设置下级数据
                    parentResultData.addChildResult(childrenResultData)
                return parentResultData
        isSimpleMatcher = re.match(self._simpleRegex, selector)
        ## 简单选择器
        if isSimpleMatcher:
            return self._findDataBySubSelector(dataRows, selector)
        ## 不识别的选择器
        raise SmartModuleExceptin("无法识别该格式，请仔细检查选择器，selector：" + selector)







if __name__ == '__main__':
    selector = SelectorMatcher()

    selectedDataResult = selector.select(game_menus, "start_game_0>5>#first")
    targetData = selectedDataResult.getTargetResult()
    selectedDataResult.printResultData()

    selectedDataResult = selector.select(game_menus, "start_game_0>5>#last")
    targetData = selectedDataResult.getTargetResult()
    selectedDataResult.printResultData()


from smart_module.module_system.module_scripts import *

'''

    据点税收增强，
    原游戏中，税收太少，导致军队士兵一直都很孱弱，没有办法和玩家进行交战，以至于玩家50个士兵可以对战200个领主士兵，
    主要原因是领主没有钱进行军队的升级和维护
    
    对于玩家来说，租金太少，没有办法对建筑进行升级和维护军队
'''

## 每天更新（24）
center_tax_inscrease_interval = 24

## 最终(2400) = 200 * 7（1400） + 1000（最大税收）
village_rent_increase_per_day = 200
## 最终(4800) = 400 * 7（2800） + 2000（最大税收）
castle_rent_increase_per_day = 400
## 最终(7600) = 800 * 7（5600） + 2000（最大税收）
town_rent_increase_per_day = 800


centerEnhanceRents = {
    "name":"CenterEnhanceRents",
    "enable":True,
    "version": "v2.0.0",
    "desc":'''
    
        据点税收增强，
        原游戏中，税收太少，导致军队士兵一直都很孱弱，没有办法和玩家进行交战，以至于玩家50个士兵可以对战200个领主士兵，
        主要原因是领主没有钱进行军队的升级和维护
        
        对于玩家来说，租金太少，没有办法对建筑进行升级和维护军队
    ''',
    "actions":[
        ("Append@simple_triggers",[
            ## 增加税收
            (center_tax_inscrease_interval, [
                (try_for_range, ":center_no", centers_begin, centers_end),
                    (party_get_slot, ":rents", ":center_no", slot_center_accumulated_rents),
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_party_type, spt_village),
                    (val_add, ":rents", village_rent_increase_per_day),
                    (else_try),
                    (party_slot_eq, ":center_no", slot_party_type, spt_castle),
                    (val_add, ":rents", castle_rent_increase_per_day),
                    (else_try),
                    (party_slot_eq, ":center_no", slot_party_type, spt_town),
                    (val_add, ":rents", town_rent_increase_per_day),
                    (try_end),
                    (party_set_slot, ":center_no", slot_center_accumulated_rents, ":rents"),

                    # (str_store_party_name,s1,":center_no"),
                    # (assign,reg1,":rents"),
                    # (display_message,"@update {s1} 's rents {reg1}"),
                (try_end),
            ]),
        ]),
    ],
}


from smart_module.module_system.module_info import *
from smart_module.module_system.module_meshes import *

from smart_module.module_system.process_common import *



def save_meshes(export_dir):
  ofile = open(export_dir + "meshes.txt","w",encoding="utf-8")
  ofile.write("%d\n"%len(meshes))
  for i_mesh in range(len(meshes)):
    mesh = meshes[i_mesh]
    ofile.write("mesh_%s %d %s %f %f %f %f %f %f %f %f %f\n"%(mesh[0],mesh[1],replace_spaces(mesh[2]),mesh[3],mesh[4],mesh[5],mesh[6],mesh[7],mesh[8],mesh[9],mesh[10],mesh[11]))
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_meshes.py","w",encoding="utf-8")
  for i_mesh in range(len(meshes)):
    ofile.write("mesh_%s = %d\n"%(meshes[i_mesh][0],i_mesh))
  ofile.write("\n\n")
  ofile.close()


def processMeshs(context):
  print( "Exporting meshes...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  save_meshes(export_dir)



from smart_module.smart_core.base.executor.smart_modules_command_executor import CommandExecutor
from smart_module.smart_core.base.executor.smart_modules_quick_command import QuickCommandParser
from smart_module.smart_core.smart_modules_log import info
from smart_module.smart_modules.center.village.VillageBasicBusiness import villageBasicBusiness


################################################
###
### 新功能-SmartModule执行器
### 用于执行actions中的快捷命令
###
################################################

class SmartModuleExecutor:
    def __init__(self):
        self.quickCommandParser = QuickCommandParser()
        self.commandExecutor = CommandExecutor()
        pass

    def collectCommandList(self, smartModule):
        moduleName = smartModule.get("name")
        version = smartModule.get("version")
        enable = smartModule.get("enable")
        moduleInfoStr = "{}（{}）".format(moduleName,version)
        if not enable:
            info("[{}] is disabled".format(moduleInfoStr))
            return
        ## 自定义指令
        customCommands = smartModule.get("commands")

        ## 与剧本相关的数据
        actions = smartModule.get("actions")

        commandList = []
        for action in actions:
            quickCommand = action[0]
            customData = action[1]
            ## 解析命令
            commandInfo = self.quickCommandParser.parseCommand(moduleInfoStr, quickCommand, customCommands, customData)
            commandList.append(commandInfo)
        return commandList



if __name__ == '__main__':
    smartModuleExecutor = SmartModuleExecutor()
    smartModuleExecutor.collectCommandList(villageBasicBusiness)






## 包含一些对阵营常用的操作

## args
from smart_module.module_system.header_common import *
from smart_module.module_system.header_operations import *
from smart_module.module_system.module_constants import *

factionBaseScripts={
    "name":"FactionBaseScripts",
    "enable":True,
    "version": "v2.0.0",
    "desc":"与阵营相关的操作",
    "actions":[
        ("Append@scripts",[
            ## 获得敌国数量
            ("get_num_of_enemy_state",[
                (store_script_param, ":faction_no", 1),
                (assign,":war_num",0),
                (try_for_range,":other_faction_no",kingdoms_begin,kingdoms_end),
                    (neq,":faction_no",":other_faction_no"),
                    (store_relation,":relation",":faction_no",":other_faction_no"),
                    # (assign,reg1,":relation"),
                    # (display_message,"@relation is {reg1}"),
                    (lt,":relation",0),
                    (val_add,":war_num",1),
                (try_end),
                (assign,reg0,":war_num"),
                # (str_store_faction_name, s1, ":faction_no"),
                # (display_message, "@faction({s1}) enemy state nums is {reg0}"),
              ]),
        ])
    ],
}


prsnt_game_credits = 0
prsnt_game_profile_banner_selection = 1
prsnt_game_custom_battle_designer = 2
prsnt_game_multiplayer_admin_panel = 3
prsnt_multiplayer_welcome_message = 4
prsnt_multiplayer_team_select = 5
prsnt_multiplayer_troop_select = 6
prsnt_multiplayer_item_select = 7
prsnt_multiplayer_message_1 = 8
prsnt_multiplayer_message_2 = 9
prsnt_multiplayer_message_3 = 10
prsnt_multiplayer_round_time_counter = 11
prsnt_multiplayer_team_score_display = 12
prsnt_multiplayer_flag_projection_display = 13
prsnt_multiplayer_flag_projection_display_bt = 14
prsnt_multiplayer_destructible_targets_display = 15
prsnt_multiplayer_respawn_time_counter = 16
prsnt_multiplayer_stats_chart = 17
prsnt_multiplayer_stats_chart_deathmatch = 18
prsnt_multiplayer_escape_menu = 19
prsnt_multiplayer_poll_menu = 20
prsnt_multiplayer_show_players_list = 21
prsnt_multiplayer_show_maps_list = 22
prsnt_multiplayer_show_factions_list = 23
prsnt_multiplayer_show_number_of_bots_list = 24
prsnt_multiplayer_poll = 25
prsnt_tutorial_show_mouse_movement = 26
prsnt_name_kingdom = 27
prsnt_banner_selection = 28
prsnt_custom_banner = 29
prsnt_banner_charge_positioning = 30
prsnt_banner_charge_selection = 31
prsnt_banner_background_selection = 32
prsnt_banner_flag_type_selection = 33
prsnt_banner_flag_map_type_selection = 34
prsnt_color_selection = 35
prsnt_marshall_selection = 36
prsnt_battle = 37
prsnt_sliders = 38
prsnt_arena_training = 39
prsnt_retirement = 40
prsnt_budget_report = 41
prsnt_game_before_quit = 42
prsnt_multiplayer_duel_start_counter = 43
